@@ -30,7 +30,6 @@ from .support.exceptions import CommandError
 from .support.protocol import NullProtocol, DryRunProtocol, \
     ExecutionTimeProtocol, ExecutionTimeExternalsProtocol
 from .utils import on_windows
-from . import cfg
 
 lgr = logging.getLogger('datalad.cmd')
 
@@ -42,6 +41,7 @@ if PY2:
     # https://pypi.python.org/pypi/subprocess32/
     pass
 
+
 class Runner(object):
     """Provides a wrapper for calling functions and commands.
 
@@ -56,9 +56,9 @@ class Runner(object):
     able to record calls and allows for dry runs.
     """
 
-    __slots__ = ['commands', 'dry', 'cwd', 'env', 'protocol']
+    __slots__ = ['commands', 'dry', 'cwd', 'env', 'protocol', '_log_outputs']
 
-    def __init__(self, cwd=None, env=None, protocol=None):
+    def __init__(self, cwd=None, env=None, protocol=None, log_outputs=None):
         """
         Parameters
         ----------
@@ -70,19 +70,22 @@ class Runner(object):
              call via env option
         protocol: ProtocolInterface
              Protocol object to write to.
+        log_outputs : bool, optional
+             Switch to instruct either outputs should be logged or not.  If not
+             set (default), config 'datalad.log outputs' would be consulted
         """
 
         self.cwd = cwd
         self.env = env
         if protocol is None:
             # TODO: config cmd.protocol = null
-            cfg = os.environ.get('DATALAD_CMD_PROTOCOL', 'null')
+            protocol_str = os.environ.get('DATALAD_CMD_PROTOCOL', 'null')
             protocol = {
                 'externals-time': ExecutionTimeExternalsProtocol,
                 'time': ExecutionTimeProtocol,
                 'null': NullProtocol
-            }[cfg]()
-            if cfg != 'null':
+            }[protocol_str]()
+            if protocol_str != 'null':
                 # we need to dump it into a file at the end
                 # TODO: config cmd.protocol_prefix = protocol
                 filename = '%s-%s.log' % (
@@ -92,6 +95,7 @@ class Runner(object):
                 atexit.register(functools.partial(protocol.write_to_file, filename))
 
         self.protocol = protocol
+        self._log_outputs = log_outputs  # we don't know yet either we need ot log every output or not
 
     def __call__(self, cmd, *args, **kwargs):
         """Convenience method
@@ -122,25 +126,77 @@ class Runner(object):
             return self.call(cmd, *args, **kwargs)
         else:
             raise TypeError("Argument 'command' is neither a string, "
-                             "nor a list nor a callable.")
+                            "nor a list nor a callable.")
+
+    @property
+    def log_outputs(self):
+        if self._log_outputs is None:
+            try:
+                from . import cfg
+                self._log_outputs = \
+                    cfg.getbool('datalad.log', 'outputs', default=False)
+            except ImportError:
+                # could be too early, then DON'T log!
+                return True
+        return self._log_outputs
 
     # Two helpers to encapsulate formatting/output
     def _log_out(self, line):
-        if line:
+        if line and self.log_outputs:
             self.log("stdout| " + line.rstrip('\n'))
 
     def _log_err(self, line, expected=False):
-        if line:
+        if line and self.log_outputs:
             self.log("stderr| " + line.rstrip('\n'),
                      level={True: logging.DEBUG,
                             False: logging.ERROR}[expected])
 
     def _get_output_online(self, proc, log_stdout, log_stderr,
                            expect_stderr=False, expect_fail=False):
+        """
+
+        If log_stdout or log_stderr are callables, they will be given a read
+        line to be processed, and return processed result.  So if they need to
+        'swallow' the line from being logged, should just return None
+
+        Parameters
+        ----------
+        proc
+        log_stdout: bool or callable or 'online' or 'offline'
+        log_stderr: : bool or callable or 'online' or 'offline'
+          If any of those 'offline', we would call proc.communicate at the
+          end to grab possibly outstanding output from it
+        expect_stderr
+        expect_fail
+
+        Returns
+        -------
+
+        """
         stdout, stderr = binary_type(), binary_type()
+
+        def decide_to_log(v):
+            """Hacky workaround for now so we could specify per each which to
+            log online and which to the log"""
+            if isinstance(v, bool) or callable(v):
+                return v
+            elif v in {'online'}:
+                return True
+            elif v in {'offline'}:
+                return False
+            else:
+                raise ValueError("can be bool, callable, 'online' or 'offline'")
+
+        log_stdout_ = decide_to_log(log_stdout)
+        log_stderr_ = decide_to_log(log_stderr)
+
         while proc.poll() is None:
-            if log_stdout:
+            if log_stdout_:
+                lgr.log(3, "Reading line from stdout")
                 line = proc.stdout.readline()
+                if line and callable(log_stdout_):
+                    # Let it be processed
+                    line = log_stdout_(line)
                 if line:
                     stdout += line
                     self._log_out(line.decode())
@@ -150,8 +206,16 @@ class Runner(object):
             else:
                 pass
 
-            if log_stderr:
+            if log_stderr_:
+                # see for a possibly useful approach to processing output
+                # in another thread http://codereview.stackexchange.com/a/17959
+                # current problem is that if there is no output on stderr
+                # it stalls
+                lgr.log(3, "Reading line from stderr")
                 line = proc.stderr.readline()
+                if line and callable(log_stderr_):
+                    # Let it be processed
+                    line = log_stderr_(line)
                 if line:
                     stderr += line
                     self._log_err(line.decode() if PY3 else line,
@@ -162,6 +226,13 @@ class Runner(object):
             else:
                 pass
 
+        if log_stdout in {'offline'} or log_stderr in {'offline'}:
+            lgr.log(4, "Issuing proc.communicate() since one of the targets "
+                       "is 'offline'")
+            stdout_, stderr_ = proc.communicate()
+            stdout += stdout_
+            stderr += stderr_
+
         return stdout, stderr
 
     def run(self, cmd, log_stdout=True, log_stderr=True, log_online=False,
@@ -231,6 +302,9 @@ class Runner(object):
            in CommandError's `stdout` and `stderr` fields respectively.
         """
 
+        # TODO:  having two PIPEs is dangerous, and leads to lock downs so we
+        # would need either threaded solution as in .communicate or just allow
+        # only one to be monitored and another one just being dumped into a file
         outputstream = subprocess.PIPE if log_stdout else sys.stdout
         errstream = subprocess.PIPE if log_stderr else sys.stderr
 
@@ -247,7 +321,6 @@ class Runner(object):
                     shlex.split(cmd, posix=not on_windows)
                     if isinstance(cmd, string_types)
                     else cmd)
-
             try:
                 proc = subprocess.Popen(cmd, stdout=outputstream,
                                         stderr=errstream,
@@ -323,9 +396,8 @@ class Runner(object):
         if self.protocol.do_execute_callables:
             if self.protocol.records_callables:
                 prot_exc = None
-                prot_id = self.protocol.start_section([str(f),
-                                                 "args=%s" % str(args),
-                                                 "kwargs=%s" % str(kwargs)])
+                prot_id = self.protocol.start_section(
+                    [str(f), "args=%s" % str(args), "kwargs=%s" % str(kwargs)])
 
             try:
                 return f(*args, **kwargs)
@@ -337,10 +409,9 @@ class Runner(object):
                     self.protocol.end_section(prot_id, prot_exc)
         else:
             if self.protocol.records_callables:
-                self.protocol.add_section([str(f),
-                                             "args=%s" % str(args),
-                                             "kwargs=%s" % str(kwargs)],
-                                          None)
+                self.protocol.add_section(
+                    [str(f), "args=%s" % str(args), "kwargs=%s" % str(kwargs)],
+                    None)
 
     def log(self, msg, level=logging.DEBUG):
         """log helper
@@ -383,6 +454,7 @@ class GitRunner(Runner):
         return super(GitRunner, self).run(
             cmd, env=self.get_git_environ_adjusted(), *args, **kwargs)
 
+
 # ####
 # Preserve from previous version
 # TODO: document intention
@@ -413,8 +485,10 @@ def link_file_load(src, dst, dry_run=False):
 
 
 def get_runner(*args, **kwargs):
+    # needs local import, because the ConfigManager itself needs the runner
+    from . import cfg
     # TODO:  this is all crawl specific -- should be moved away
-    if cfg.getboolean('crawl', 'dryrun', default=False):
+    if cfg.obtain('datalad.crawl.dryrun', default=False):
         kwargs = kwargs.copy()
         kwargs['protocol'] = DryRunProtocol()
     return Runner(*args, **kwargs)
