@@ -15,29 +15,44 @@ import argparse
 import os
 import re
 import sys
-
+import gzip
 from tempfile import NamedTemporaryFile
 
 from ..cmd import Runner
 from ..log import is_interactive
 from ..utils import getpwd
+from ..version import __version__
+from ..dochelpers import exc_str
 
 from logging import getLogger
 lgr = getLogger('datalad.cmdline')
 
+
 class HelpAction(argparse.Action):
     def __call__(self, parser, namespace, values, option_string=None):
         if is_interactive() and option_string == '--help':
             # lets use the manpage on mature systems ...
             try:
                 import subprocess
+                # get the datalad manpage to use
+                manfile = os.environ.get('MANPATH', '/usr/share/man') \
+                    + '/man1/{0}.1.gz'.format(parser.prog.replace(' ', '-'))
+                # extract version field from the manpage
+                if not os.path.exists(manfile):
+                    raise IOError("manfile is not found")
+                with gzip.open(manfile) as f:
+                    man_th = [line for line in f if line.startswith(b".TH")][0]
+                man_version = man_th.split(b' ')[5].strip(b" '\"\t\n").decode('utf-8')
+
+                # don't show manpage if man_version not equal to current datalad_version
+                if __version__ != man_version:
+                    raise ValueError
                 subprocess.check_call(
-                    'man %s 2> /dev/null' % parser.prog.replace(' ', '-'),
+                    'man %s 2> /dev/null' % manfile,
                     shell=True)
                 sys.exit(0)
-            except (subprocess.CalledProcessError, OSError):
-                # ...but silently fall back if it doesn't work
-                pass
+            except (subprocess.CalledProcessError, IOError, OSError, IndexError, ValueError) as e:
+                lgr.debug("Did not use manpage since %s", exc_str(e))
         if option_string == '-h':
             helpstr = "%s\n%s" % (
                 parser.format_usage(),
@@ -81,27 +96,14 @@ class LogLevelAction(argparse.Action):
         from ..log import LoggerHelper
         LoggerHelper().set_level(level=values)
 
-class PBSAction(argparse.Action):
-    """Action to schedule actual command execution via PBS (e.g. Condor)"""
-    def __call__(self, parser, namespace, values, option_string=None):
-        pbs = values[0]
-        import pdb; pdb.set_trace()
-        i = 1
 
-
-def parser_add_common_args(parser, pos=None, opt=None, **kwargs):
-    from . import common_args
-    for i, args in enumerate((pos, opt)):
-        if args is None:
-            continue
-        for arg in args:
-            arg_tmpl = getattr(common_args, arg)
-            arg_kwargs = arg_tmpl[2].copy()
-            arg_kwargs.update(kwargs)
-            if i:
-                parser.add_argument(*arg_tmpl[i], **arg_kwargs)
-            else:
-                parser.add_argument(arg_tmpl[i], **arg_kwargs)
+# MIH: Disabled. Non-functional, untested.
+#class PBSAction(argparse.Action):
+#    """Action to schedule actual command execution via PBS (e.g. Condor)"""
+#    def __call__(self, parser, namespace, values, option_string=None):
+#        pbs = values[0]
+#        import pdb; pdb.set_trace()
+#        i = 1
 
 
 def parser_add_common_opt(parser, opt, names=None, **kwargs):
@@ -128,7 +130,7 @@ def strip_arg_from_argv(args, value, opt_names):
             # we skip only one as instructed
             skip -= 1
             continue
-        if not (arg in opt_names and i < len(args)-1 and args[i + 1] == value):
+        if not (arg in opt_names and i < len(args) - 1 and args[i + 1] == value):
             args_clean.append(arg)
         else:
             # we need to skip this one and next one
@@ -164,6 +166,7 @@ queue
     finally:
         os.unlink(f.name)
 
+
 class RegexpType(object):
     """Factory for creating regular expression types for argparse
 
@@ -200,8 +203,7 @@ def get_repo_instance(path=curdir, class_=None):
     RuntimeError, in case cwd is not inside a known repository.
     """
 
-    from os.path import join as opj, ismount, exists, abspath, expanduser, \
-        expandvars, normpath, isabs
+    from os.path import ismount, exists, normpath, isabs
     from git.exc import InvalidGitRepositoryError
     from ..utils import expandpath
     from ..support.gitrepo import GitRepo
@@ -249,4 +251,4 @@ def get_repo_instance(path=curdir, class_=None):
 from appdirs import AppDirs
 from os.path import join as opj
 
-dirs = AppDirs("datalad", "datalad.org")
\ No newline at end of file
+dirs = AppDirs("datalad", "datalad.org")
