@@ -10,19 +10,21 @@
 
 __docformat__ = 'restructuredtext'
 
+import logging
+lgr = logging.getLogger('datalad.cmdline')
+
+lgr.log(5, "Importing cmdline.main")
 
 import argparse
-import logging
 import sys
-import os
 import textwrap
 from importlib import import_module
 
 import datalad
-from datalad.log import lgr
 
 from datalad.cmdline import helpers
 from datalad.support.exceptions import InsufficientArgumentsError
+from datalad.support.exceptions import IncompleteResultsError
 from ..utils import setup_exceptionhook, chpwd
 from ..dochelpers import exc_str
 
@@ -51,9 +53,16 @@ THE SOFTWARE.
 """
 
 
+# TODO:  OPT look into making setup_parser smarter to become faster
+# Now it seems to take up to 200ms to do all the parser setup
+# even though it might not be necessary to know about all the commands etc.
+# I wondered if it could somehow decide on what commands to worry about etc
+# by going through sys.args first
 def setup_parser(
         formatter_class=argparse.RawDescriptionHelpFormatter,
         return_subparsers=False):
+
+    lgr.log(5, "Starting to setup_parser")
     # delay since it can be a heavy import
     from ..interface.base import dedent_docstring, get_interface_groups, \
         get_cmdline_command_name, alter_interface_docs_for_cmdline
@@ -61,7 +70,9 @@ def setup_parser(
     parts = {}
     # main parser
     parser = argparse.ArgumentParser(
-        fromfile_prefix_chars='@',
+        # cannot use '@' because we need to input JSON-LD properties (which might come wit @ prefix)
+        # MH: question, do we need this at all?
+        fromfile_prefix_chars=':',
         # usage="%(prog)s ...",
         description=dedent_docstring("""\
             DataLad provides a unified data distribution with the convenience of git-annex
@@ -81,7 +92,10 @@ def setup_parser(
     if __debug__:
         parser.add_argument(
             '--dbg', action='store_true', dest='common_debug',
-            help="do not catch exceptions and show exception traceback")
+            help="enter Python debugger when uncaught exception happens")
+        parser.add_argument(
+            '--idbg', action='store_true', dest='common_idebug',
+            help="enter IPython debugger when uncaught exception happens")
     parser.add_argument(
         '-C', action='append', dest='change_path', metavar='PATH',
         help="""run as if datalad was started in <path> instead
@@ -118,6 +132,7 @@ def setup_parser(
 
         for _intfspec in _interfaces:
             # turn the interface spec into an instance
+            lgr.log(5, "Importing module %s " % _intfspec[0])
             _mod = import_module(_intfspec[0], package='datalad')
             _intf = getattr(_mod, _intfspec[1])
             cmd_name = get_cmdline_command_name(_intfspec)
@@ -128,7 +143,7 @@ def setup_parser(
                 parser_args = dict(formatter_class=formatter_class)
             # use class description, if no explicit description is available
                 parser_args['description'] = alter_interface_docs_for_cmdline(
-                    _intf.__doc__)
+                    _intf.__doc__.strip())
             # create subparser, use module suffix as cmd name
             subparser = subparsers.add_parser(cmd_name, add_help=False, **parser_args)
             # all subparser can report the version
@@ -167,13 +182,13 @@ def setup_parser(
 
         cmd_summary.append('\n*%s*\n' % (grp_descr,))
         for cd in grp_cmds:
-            cmd_summary.append('  - %s:  %s'
-                               % (cd[0],
+            cmd_summary.append('  - %-20s %s'
+                               % ((cd[0] + ':',
                                   textwrap.fill(
                                       cd[1].rstrip(' .'),
                                       75,
                                       #initial_indent=' ' * 4,
-                                      subsequent_indent=' ' * 8)))
+                                      subsequent_indent=' ' * 8))))
     # we need one last formal section to not have the trailed be
     # confused with the last command group
     cmd_summary.append('\n*General information*\n')
@@ -186,6 +201,7 @@ def setup_parser(
     datalad <command> --help"""),
                          75, initial_indent='', subsequent_indent=''))
     parts['datalad'] = parser
+    lgr.log(5, "Finished setup_parser")
     if return_subparsers:
         return parts
     else:
@@ -203,6 +219,7 @@ def setup_parser(
 
 
 def main(args=None):
+    lgr.log(5, "Starting main(%r)", args)
     # PYTHON_ARGCOMPLETE_OK
     parser = setup_parser()
     try:
@@ -212,8 +229,20 @@ def main(args=None):
         pass
 
     # parse cmd args
-    cmdlineargs = parser.parse_args(args)
-    if not cmdlineargs.change_path is None:
+    cmdlineargs, unparsed_args = parser.parse_known_args(args)
+    has_func = hasattr(cmdlineargs, 'func') and cmdlineargs.func is not None
+    if unparsed_args:
+        if has_func and cmdlineargs.func.__self__.__name__ != 'Export':
+            lgr.error('unknown argument{}: {}'.format(
+                's' if len(unparsed_args) > 1 else '',
+                unparsed_args if len(unparsed_args) > 1 else unparsed_args[0]))
+            cmdlineargs.subparser.print_usage()
+            sys.exit(1)
+        else:
+            # store all unparsed arguments
+            cmdlineargs.datalad_unparsed_args = unparsed_args
+
+    if cmdlineargs.change_path is not None:
         for path in cmdlineargs.change_path:
             chpwd(path)
 
@@ -225,22 +254,50 @@ def main(args=None):
         args_ = strip_arg_from_argv(args or sys.argv, cmdlineargs.pbs_runner, pbs_runner_opt[1])
         # run the function associated with the selected command
         run_via_pbs(args_, cmdlineargs.pbs_runner)
-    elif cmdlineargs.common_debug:
-        # so we could see/stop clearly at the point of failure
-        setup_exceptionhook()
-        ret = cmdlineargs.func(cmdlineargs)
-    else:
-        # otherwise - guard and only log the summary. Postmortem is not
-        # as convenient if being caught in this ultimate except
-        try:
+    elif has_func:
+        if cmdlineargs.common_debug or cmdlineargs.common_idebug:
+            # so we could see/stop clearly at the point of failure
+            setup_exceptionhook(ipython=cmdlineargs.common_idebug)
             ret = cmdlineargs.func(cmdlineargs)
-        except InsufficientArgumentsError as exc:
-            # if the func reports inappropriate usage, give help output
-            lgr.error('%s (%s)' % (exc_str(exc), exc.__class__.__name__))
-            cmdlineargs.subparser.print_usage()
-            sys.exit(1)
-        except Exception as exc:
-            lgr.error('%s (%s)' % (exc_str(exc), exc.__class__.__name__))
-            sys.exit(1)
-    if hasattr(cmdlineargs, 'result_renderer'):
-        cmdlineargs.result_renderer(ret)
+        else:
+            # otherwise - guard and only log the summary. Postmortem is not
+            # as convenient if being caught in this ultimate except
+            try:
+                ret = cmdlineargs.func(cmdlineargs)
+            except InsufficientArgumentsError as exc:
+                # if the func reports inappropriate usage, give help output
+                lgr.error('%s (%s)' % (exc_str(exc), exc.__class__.__name__))
+                cmdlineargs.subparser.print_usage(sys.stderr)
+                sys.exit(2)
+            except IncompleteResultsError as exc:
+                # we didn't get everything we wanted: still try to present what
+                # we got as usual, but exit with an error
+                try:
+                    if hasattr(cmdlineargs, 'result_renderer'):
+                        cmdlineargs.result_renderer(exc.results, cmdlineargs)
+                except Exception as exc2:
+                    # could be anything really, but we do not want to blow
+                    #  loud in exception handler
+                    lgr.warning("Failed to render partial results: %s", exc_str(exc2))
+                lgr.error('could not perform all requested actions: %s',
+                          exc_str(exc))
+                sys.exit(1)
+            except Exception as exc:
+                lgr.error('%s (%s)' % (exc_str(exc), exc.__class__.__name__))
+                sys.exit(1)
+    else:
+        # just let argparser spit out its error, since there is smth wrong
+        parser.parse_args(args)
+        # if that one didn't puke -- we should
+        parser.print_usage()
+        lgr.error("Please specify the command")
+        sys.exit(2)
+
+    try:
+        if hasattr(cmdlineargs, 'result_renderer'):
+            cmdlineargs.result_renderer(ret, cmdlineargs)
+    except Exception as exc:
+        lgr.error("Failed to render results due to %s", exc_str(exc))
+        sys.exit(1)
+
+lgr.log(5, "Done importing cmdline.main")
