@@ -53,7 +53,6 @@ from glob import glob
 from os.path import dirname, join as opj, isabs, exists, curdir, basename
 from os import makedirs
 
-from .. import cfg
 from ..consts import CRAWLER_META_DIR, HANDLE_META_DIR, CRAWLER_META_CONFIG_PATH
 from ..consts import CRAWLER_META_CONFIG_FILENAME
 from ..utils import updated
@@ -61,6 +60,7 @@ from ..dochelpers import exc_str
 from ..support.gitrepo import GitRepo
 from ..support.network import parse_url_opts
 from ..support.stats import ActivityStats
+from ..support.exceptions import PipelineNotSpecifiedError
 from ..support.configparserinc import SafeConfigParserWithIncludes
 
 from logging import getLogger
@@ -70,6 +70,7 @@ lgr = getLogger('datalad.crawler.pipeline')
 CRAWLER_PIPELINE_SECTION = 'crawl:pipeline'
 CRAWLER_PIPELINE_SECTION_DEPRECATED = 'crawler'
 
+
 class FinishPipeline(Exception):
     """Exception to use to signal that any given pipeline should be stopped
     """
@@ -313,7 +314,7 @@ def _compare_dicts(d1, d2):
                 try:
                     if d1[k] != d2[k]:
                         changed.append(k)
-                except:
+                except:  # MIH: TypeError?
                     maybe_changed.append(k)
     return added, changed, removed, maybe_changed
 
@@ -349,14 +350,15 @@ def initiate_pipeline_config(template, template_func=None, template_kwargs=None,
         repo = GitRepo(path)
         repo.add(crawl_config_repo_path)
         if repo.dirty:
-            repo.commit("Initialized crawling configuration to use template %s" % template)
+            repo.commit("Initialized crawling configuration to use template %s" % template,
+                        _datalad_msg=True)
         else:
             lgr.debug("Repository is not dirty -- not committing")
 
     return crawl_config
 
 
-def load_pipeline_from_module(module, func=None, args=None, kwargs=None):
+def load_pipeline_from_module(module, func=None, args=None, kwargs=None, return_only=False):
     """Load pipeline from a Python module
 
     Parameters
@@ -369,6 +371,8 @@ def load_pipeline_from_module(module, func=None, args=None, kwargs=None):
       Positional arguments to provide to the function.
     kwargs: dict, optional
       Keyword arguments to provide to the function.
+    return_only: bool, optional
+      flag true if only to return pipeline
     """
 
     func = func or 'pipeline'
@@ -387,6 +391,8 @@ def load_pipeline_from_module(module, func=None, args=None, kwargs=None):
                              fromlist=['datalad.crawler.pipelines'])
         else:
             mod = __import__(modname, level=0)
+        if return_only:
+            return getattr(mod, func)
         return getattr(mod, func)(*args, **kwargs)
     except Exception as e:
         raise RuntimeError("Failed to import pipeline from %s: %s" % (module, exc_str(e)))
@@ -424,7 +430,7 @@ def _find_pipeline(name):
     return None
 
 
-def load_pipeline_from_template(name, func=None, args=None, kwargs=None):
+def load_pipeline_from_template(name, func=None, args=None, kwargs=None, return_only=False):
     """Given a name, loads that pipeline from datalad.crawler.pipelines
 
     and later from other locations
@@ -432,11 +438,18 @@ def load_pipeline_from_template(name, func=None, args=None, kwargs=None):
     Parameters
     ----------
     name: str
-        Name of the pipeline defining the filename, or the full path to it (TODO)
+        Name of the pipeline (the template) defining the filename, or the full path to it (TODO),
+        example: openfmri
+    func: str
+        Name of function from which pipeline to run
+        example: superdataset_pipeline
     args: dict, optional
         Positional args for the pipeline, passed as `*args` into the pipeline call
     kwargs: dict, optional
-        Keyword args for the pipeline, passed as `**kwargs` into the pipeline call
+        Keyword args for the pipeline, passed as `**kwargs` into the pipeline call,
+        example: {'dataset': 'ds000001'}
+    return_only: bool, optional
+        flag true if only to return pipeline
     """
 
     if isabs(name) or exists(name):
@@ -449,11 +462,11 @@ def load_pipeline_from_template(name, func=None, args=None, kwargs=None):
 
     if filename:
         if not exists(filename):
-            raise IOError("Pipeline file %s is N/A" % filename)
+            raise PipelineNotSpecifiedError("Pipeline file %s is N/A" % filename)
     else:
-        raise ValueError("could not find pipeline for %s" % name)
+        raise PipelineNotSpecifiedError("could not find pipeline for %s" % name)
 
-    return load_pipeline_from_module(filename, func=func, args=args, kwargs=kwargs)
+    return load_pipeline_from_module(filename, func=func, args=args, kwargs=kwargs, return_only=return_only)
 
 
 # TODO: we might need to find present .datalad/crawl in another branch if not
@@ -462,20 +475,20 @@ def load_pipeline_from_template(name, func=None, args=None, kwargs=None):
 
 def load_pipeline_from_config(path):
     """Given a path to the pipeline configuration file, instantiate a pipeline
-    
+
     Typical example description
-    
+
         [crawl:pipeline]
         pipeline = standard
         func = pipeline1
         _kwarg1 = 1
-   
+
     which would instantiate a pipeline from standard.py module by calling
     `standard.pipeline1` with `_kwarg1='1'`.  This definition is identical to
-    
+
         [crawl:pipeline]
         pipeline = standard?func=pipeline1&_kwarg1=1
-   
+
     so that theoretically we could specify basic pipelines completely within
     a URL
     """
@@ -492,7 +505,7 @@ def load_pipeline_from_config(path):
         opts = cfg_.options(sec)
         # must have template
         if 'template' not in opts:
-            raise IOError("%s lacks %r field within %s section" % (path, '_template', sec))
+            raise PipelineNotSpecifiedError("%s lacks %r field within %s section" % (path, 'template', sec))
         template = cfg_.get(sec, 'template')
         # parse template spec
         template_name, url_opts = parse_url_opts(template)
