@@ -8,6 +8,8 @@
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 """Base classes to custom git-annex remotes (e.g. extraction from archives)"""
 
+from __future__ import absolute_import
+
 __docformat__ = 'restructuredtext'
 
 import errno
@@ -17,12 +19,13 @@ import sys
 from os.path import exists, join as opj, realpath, dirname, lexists
 
 from six.moves import range
-from six.moves.urllib.parse import urlparse, urlunparse
+from six.moves.urllib.parse import urlparse
 
 import logging
 lgr = logging.getLogger('datalad.customremotes')
 lgr.log(5, "Importing datalad.customremotes.main")
 
+from ..ui import ui
 from ..support.protocol import ProtocolInterface
 from ..support.cache import DictCache
 from ..cmdline.helpers import get_repo_instance
@@ -34,6 +37,8 @@ SUPPORTED_PROTOCOL = 1
 DEFAULT_COST = 100
 DEFAULT_AVAILABILITY = "local"
 
+from datalad.ui.progressbars import ProgressBarBase
+
 
 class AnnexRemoteQuit(Exception):
     pass
@@ -103,8 +108,9 @@ send () {
             return  # nothing else to be done
 
         lgr.debug("Initiating protocoling."
-                      "cd %s; vim %s"
-                      % (realpath(self.repopath), _file[len(self.repopath)+1:]))
+                  "cd %s; vim %s"
+                  % (realpath(self.repopath),
+                     _file[len(self.repopath) + 1:]))
         with open(_file, 'a') as f:
             f.write(self.HEADER)
         os.chmod(_file, 0o755)
@@ -114,10 +120,10 @@ send () {
         with open(self._file, 'a') as f:
             f.write('%s### %s%s' % (os.linesep, cmd, os.linesep))
         lgr.debug("New section in the protocol: "
-                      "cd %s; PATH=%s:$PATH %s"
-                      % (realpath(self.repopath),
-                         dirname(self._file),
-                         cmd))
+                  "cd %s; PATH=%s:$PATH %s"
+                  % (realpath(self.repopath),
+                     dirname(self._file),
+                     cmd))
 
     def write_entries(self, entries):
         self.initiate()
@@ -172,9 +178,10 @@ class AnnexCustomRemote(object):
     CUSTOM_REMOTE_NAME = None  # if None -- no additional custom remote name
     # SUPPORTED_SCHEMES = ()
 
+    COST = DEFAULT_COST
     AVAILABILITY = DEFAULT_AVAILABILITY
 
-    def __init__(self, path=None, cost=DEFAULT_COST):  # , availability=DEFAULT_AVAILABILITY):
+    def __init__(self, path=None, cost=None):  # , availability=DEFAULT_AVAILABILITY):
         """
         Parameters
         ----------
@@ -202,18 +209,23 @@ class AnnexCustomRemote(object):
         self.path = self.repo.path
 
         self._progress = 0  # transmission to be reported back if available
+        if cost is None:
+            cost = self.COST
         self.cost = cost
         #self.availability = availability.upper()
         assert(self.AVAILABILITY.upper() in ("LOCAL", "GLOBAL"))
 
         # To signal either we are in the loop and e.g. could correspond to annex
         self._in_the_loop = False
-        self._protocol = AnnexExchangeProtocol(self.path, self.CUSTOM_REMOTE_NAME) \
-                         if os.environ.get('DATALAD_PROTOCOL_REMOTE') \
-                         else None
+        self._protocol = \
+            AnnexExchangeProtocol(self.path, self.CUSTOM_REMOTE_NAME) \
+            if os.environ.get('DATALAD_TESTS_PROTOCOLREMOTE') else None
 
         self._contentlocations = DictCache(size_limit=100)  # TODO: config ?
 
+        # instruct annex backend UI to use this remote
+        if ui.backend == 'annex':
+            ui.set_specialremote(self)
 
     @classmethod
     def _get_custom_scheme(cls, prefix):
@@ -229,8 +241,8 @@ class AnnexCustomRemote(object):
 
     # Helpers functionality
 
-    def get_contentlocation(self, key):
-        """Return absolute path to the file containing the key
+    def get_contentlocation(self, key, absolute=False, verify_exists=True):
+        """Return (relative to top or absolute) path to the file containing the key
 
         This is a wrapper around AnnexRepo.get_contentlocation which provides caching
         of the result (we are asking the location for the same archive key often)
@@ -242,11 +254,15 @@ class AnnexCustomRemote(object):
         else:
             fpath = self._contentlocations[key]
             # but verify that it exists
-            if not lexists(opj(self.path, fpath)):
+            if verify_exists and not lexists(opj(self.path, fpath)):
                 # prune from cache
                 del self._contentlocations[key]
                 fpath = ''
-        return fpath
+
+        if absolute and fpath:
+            return opj(self.path, fpath)
+        else:
+            return fpath
 
     #
     # Communication with git-annex
@@ -262,11 +278,11 @@ class AnnexCustomRemote(object):
         msg = " ".join(map(str, args))
         if not self._in_the_loop:
             lgr.debug("We are not yet in the loop, thus should not send to annex"
-                     " anything.  Got: %s" % msg.encode())
+                      " anything.  Got: %s" % msg.encode())
             return
         try:
             self.heavydebug("Sending %r" % msg)
-            self.fout.write(msg + "\n")#.encode())
+            self.fout.write(msg + "\n")  # .encode())
             self.fout.flush()
             if self._protocol is not None:
                 self._protocol += "send %s" % msg
@@ -280,7 +296,6 @@ class AnnexCustomRemote(object):
     def send_unsupported(self):
         self.send("UNSUPPORTED-REQUEST")
 
-
     def read(self, req=None, n=1):
         """Read a message from git-annex
 
@@ -301,15 +316,15 @@ class AnnexCustomRemote(object):
         msg = l.split(None, n)
         if req and (req != msg[0]):
             # verify correct response was given
-            self.error("Expected %r, got %r.  Ignoring" % (resp, msg[0]))
+            self.error("Expected %r, got %r.  Ignoring" % (req, msg[0]))
             return None
         self.heavydebug("Received %r" % (msg,))
         return msg
 
     # TODO: see if we could adjust the "originating" file:line, because
     # otherwise they are all reported from main.py:117 etc
-    def heavydebug(self, msg):
-        lgr.log(4, msg)
+    def heavydebug(self, msg, *args, **kwargs):
+        lgr.log(4, msg, *args, **kwargs)
 
     # Since protocol allows for some messaging back, let's duplicate to lgr
     def debug(self, msg):
@@ -320,11 +335,10 @@ class AnnexCustomRemote(object):
         lgr.error(msg)
         self.send(annex_err, msg)
 
-    def progress(self, perc):
-        perc = int(perc)
-        if self._progress != perc:
-            self.send("PROGRESS", perc)
-
+    def progress(self, bytes):
+        bytes = int(bytes)
+        if self._progress != bytes:
+            self.send("PROGRESS", bytes)
 
     def main(self):
         """Interface to the command line tool"""
@@ -333,7 +347,7 @@ class AnnexCustomRemote(object):
             self._in_the_loop = True
             self._loop()
         except AnnexRemoteQuit:
-            pass # no harm
+            pass  # no harm
         except KeyboardInterrupt:
             self.stop("Interrupted by user")
         except Exception as e:
@@ -341,13 +355,11 @@ class AnnexCustomRemote(object):
         finally:
             self._in_the_loop = False
 
-
     def stop(self, msg=None):
         lgr.debug("Stopping communications of %s%s" %
                  (self, ": %s" % msg if msg else ""))
         raise AnnexRemoteQuit(msg)
 
-
     def _loop(self):
         """The main loop
         """
@@ -535,7 +547,8 @@ class AnnexCustomRemote(object):
                     urls.append(url[0])
                 else:
                     break
-        self.heavydebug("Received URLS: %s" % urls)
+
+        self.heavydebug("Got %d URL(s) for key %s: %s", len(urls), key, urls)
 
         if not urls:
             raise ValueError("Did not get any URLs for %s which we support" % key)
