@@ -12,8 +12,7 @@
 import os
 from os.path import join as opj, abspath, basename
 from ..dataset import Dataset
-from datalad.api import update, install
-from datalad.distribution.install import get_containing_subdataset
+from datalad.api import update, install, uninstall
 from datalad.utils import chpwd
 from datalad.support.gitrepo import GitRepo
 from datalad.support.annexrepo import AnnexRepo
@@ -24,10 +23,10 @@ from datalad.tests.utils import with_tempfile, assert_in, with_tree,\
 from datalad.tests.utils import SkipTest
 from datalad.tests.utils import assert_cwd_unchanged, skip_if_on_windows
 from datalad.tests.utils import assure_dict_from_str, assure_list_from_str
-from datalad.tests.utils import ok_generator
+from datalad.tests.utils import create_tree
 from datalad.tests.utils import assert_not_in
 from datalad.tests.utils import assert_raises
-from datalad.tests.utils import ok_startswith
+from datalad.tests.utils import ok_file_has_content
 from datalad.tests.utils import skip_if_no_module
 from datalad.tests.utils import ok_clean_git, swallow_outputs
 
@@ -38,23 +37,13 @@ from datalad.tests.utils import ok_clean_git, swallow_outputs
 def test_update_simple(origin, src_path, dst_path):
 
     # prepare src
-    source = install(path=src_path, source=origin, recursive=True)
-    # TODO: For now, circumnavigate the detached head issue.
-    # Figure out, what to do.
-    for subds in source.get_subdatasets(recursive=True):
-        AnnexRepo(opj(src_path, subds), init=True,
-                  create=True).checkout("master")
+    source = install(src_path, source=origin, recursive=True)[0]
     # forget we cloned it (provide no 'origin' anymore), which should lead to
     # setting tracking branch to target:
     source.repo.remove_remote("origin")
 
     # get a clone to update later on:
-    dest = install(path=dst_path, source=src_path, recursive=True)
-    # TODO: For now, circumnavigate the detached head issue.
-    # Figure out, what to do.
-    for subds in dest.get_subdatasets(recursive=True):
-        AnnexRepo(opj(dst_path, subds), init=True,
-                  create=True).checkout("master")
+    dest = install(dst_path, source=src_path, recursive=True)[0]
     # test setup done;
     # assert all fine
     ok_clean_git(dst_path)
@@ -68,7 +57,7 @@ def test_update_simple(origin, src_path, dst_path):
     # modify origin:
     with open(opj(src_path, "update.txt"), "w") as f:
         f.write("Additional content")
-    source.install(path="update.txt")
+    source.add(path="update.txt")
     source.save("Added update.txt")
     ok_clean_git(src_path)
 
@@ -89,9 +78,23 @@ def test_update_simple(origin, src_path, dst_path):
     dest.repo.get_file_key("update.txt")  # raises if unknown
     eq_([False], dest.repo.file_has_content(["update.txt"]))
 
+    # smoke-test if recursive update doesn't fail if submodule is removed
+    dest.uninstall('subm 1', kill=True)
+    dest.update(recursive=True)
+    dest.update(merge=True, recursive=True)
+
+    # and now test recursive update with merging in differences
+    create_tree(opj(source.path, 'subm 2'), {'load.dat': 'heavy'})
+    source.save(message="saving changes within subm2",
+                recursive=True, auto_add_changes=True)
+    dest.update(merge=True, recursive=True)
+    # and now we can get new file
+    dest.get('subm 2/load.dat')
+    ok_file_has_content(opj(dest.path, 'subm 2', 'load.dat'), 'heavy')
+
 
 def test_update_recursive():
-    raise SkipTest("TODO")
+    raise SkipTest("TODO more tests to add to above ones")
 
 
 @with_testrepos('.*annex.*', flavors=['clone'])
