@@ -19,15 +19,14 @@ from os.path import join as opj
 from datalad.interface.base import Interface
 from datalad.support.constraints import EnsureStr
 from datalad.support.constraints import EnsureNone
-from datalad.support.exceptions import CommandError
 from datalad.support.gitrepo import GitRepo
 from datalad.support.param import Parameter
 from datalad.utils import knows_annex
-from datalad.utils import getpwd
 
 from .dataset import Dataset
 from .dataset import EnsureDataset
 from .dataset import datasetmethod
+from .dataset import require_dataset
 
 lgr = logging.getLogger('datalad.distribution.update')
 
@@ -86,35 +85,27 @@ class Update(Interface):
                                       "implemented yet.")
 
         # shortcut
-        ds = dataset
-
-        if ds is not None and not isinstance(ds, Dataset):
-            ds = Dataset(ds)
-
-        # if we have no dataset given, figure out which one we need to operate
-        # on, based on the current working directory of the process:
-        if ds is None:
-            # try to find a dataset at or above PWD:
-            dspath = GitRepo.get_toppath(getpwd())
-            if dspath is None:
-                raise ValueError("No dataset found at %s." % getpwd())
-            ds = Dataset(dspath)
-        assert (ds is not None)
-
-        if not ds.is_installed():
-            raise ValueError("No installed dataset found at "
-                             "{0}.".format(ds.path))
+        ds = require_dataset(dataset, check_installed=True, purpose='updating')
         assert (ds.repo is not None)
 
         repos_to_update = [ds.repo]
         if recursive:
-            repos_to_update += [GitRepo(opj(ds.path, sub_path))
+            repos_to_update += [GitRepo(opj(ds.path, sub_path), create=False)
                                 for sub_path in
-                                ds.get_subdatasets(recursive=True)]
+                                ds.get_subdatasets(recursive=True, fulfilled=True)]
+        # only work on those which are installed
 
+
+        # TODO: current implementation disregards submodules organization,
+        #  it just updates/merge each one individually whenever in the simplest
+        #  case we just need  a call to
+        # git submodule update --recursive
+        #  if name was not provided, and there is no --merge
+        # If we do --merge we should at the end call save
         for repo in repos_to_update:
-            # get all remotes:
-            remotes = repo.get_remotes()
+            # get all remotes which have references (would exclude
+            # special remotes)
+            remotes = repo.get_remotes(with_refs_only=True)
             if name and name not in remotes:
                 lgr.warning("'%s' not known to dataset %s.\nSkipping" %
                             (name, repo.path))
@@ -141,12 +132,13 @@ class Update(Interface):
             # TODO: This isn't correct. `fetch_all` fetches all remotes.
             # Apparently, we currently fetch an entire remote anyway. Is this
             # what we want? Do we want to specify a refspec instead?
-
+            # yoh: we should leave it to git and its configuration.
+            # So imho we should just extract to fetch everything git would fetch
             if knows_annex(repo.path) and not fetch_all:
                 if name:
                     # we are updating from a certain remote, so git-annex branch
                     # should be updated from there as well:
-                    repo.fetch(remote=name, refspec="git-annex")
+                    repo.fetch(remote=name)
                     # TODO: what does failing here look like?
                 else:
                     # we have no remote given, therefore
@@ -155,7 +147,7 @@ class Update(Interface):
                     track_remote, track_branch = repo.get_tracking_branch()
                     if track_remote:
                         # we have a "tracking remote"
-                        repo.fetch(remote=track_remote, refspec="git-annex")
+                        repo.fetch(remote=track_remote)
 
             # merge:
             if merge:
