@@ -9,12 +9,11 @@
 """Provide access to stuff (html, data files) via HTTP and HTTPS
 
 """
-import functools
 import re
 import requests
 import requests.auth
 
-from six import PY3
+import io
 from six import BytesIO
 
 from ..utils import assure_list_from_str, assure_dict_from_str
@@ -30,13 +29,20 @@ from ..support.cookies import cookies_db
 from ..support.status import FileStatus
 
 from .base import Authenticator
-from .base import BaseDownloader
+from .base import BaseDownloader, DownloaderSession
 from .base import DownloadError, AccessDeniedError, AccessFailedError, UnhandledRedirectError
 
 from logging import getLogger
 from ..log import LoggerHelper
 lgr = getLogger('datalad.http')
 
+try:
+    import requests_ftp
+    requests_ftp.monkeypatch_session()
+except ImportError as e:
+    lgr.debug("Failed to import requests_ftp, thus no ftp support: %s" % exc_str(e))
+    pass
+
 if lgr.getEffectiveLevel() <= 1:
     # Let's also enable requests etc debugging
 
@@ -74,7 +80,8 @@ def check_response_status(response, err_prefix="", session=None):
         raise AccessDeniedError(err_msg)
     elif response.status_code in {200}:
         pass
-    elif response.status_code in {301, 307}:
+    elif response.status_code in {301, 302, 307}:
+        # TODO: apparently tests do not excercise this one yet
         if session is None:
             raise AccessFailedError(err_msg + " no session was provided")
         redirs = list(session.resolve_redirects(response, response.request))
@@ -91,7 +98,8 @@ def check_response_status(response, err_prefix="", session=None):
 class HTTPBaseAuthenticator(Authenticator):
     """Base class for html_form and http_auth authenticators
     """
-    def __init__(self, url=None, failure_re=None, success_re=None, **kwargs):
+    def __init__(self, url=None, failure_re=None, success_re=None,
+                 session_cookies=None, **kwargs):
         """
         Parameters
         ----------
@@ -102,12 +110,14 @@ class HTTPBaseAuthenticator(Authenticator):
         success_re : str or list of str, optional
           Regular expressions to determine either login has failed or succeeded.
           TODO: we might condition when it gets ran
+        session_cookies : str or list of str, optional
+          Session cookies to store (besides auth response cookies)
         """
         super(HTTPBaseAuthenticator, self).__init__(**kwargs)
         self.url = url
         self.failure_re = assure_list_from_str(failure_re)
         self.success_re = assure_list_from_str(success_re)
-
+        self.session_cookies = assure_list_from_str(session_cookies)
 
     def authenticate(self, url, credential, session, update=False):
         # we should use specified URL for this authentication first
@@ -124,7 +134,16 @@ class HTTPBaseAuthenticator(Authenticator):
         response = self._post_credential(credentials, post_url, session)
 
         err_prefix = "Authentication to %s failed: " % post_url
-        check_response_status(response, err_prefix, session=session)
+        try:
+            check_response_status(response, err_prefix, session=session)
+        except DownloadError:
+            # It might have happened that the return code was 'incorrect'
+            # and we did get some feedback, which we could analyze to
+            # figure out actual problem.  E.g. in case of nersc of crcns
+            # it returns 404 (not found) with text in the html
+            if response is not None and response.text:
+                self.check_for_auth_failure(response.text, err_prefix)
+            raise
 
         response_text = response.text
         self.check_for_auth_failure(response_text, err_prefix)
@@ -138,14 +157,22 @@ class HTTPBaseAuthenticator(Authenticator):
                         err_prefix + " returned output did not match 'success' regular expression %s" % success_re
                     )
 
+        cookies_dict = {}
         if response.cookies:
             cookies_dict = requests.utils.dict_from_cookiejar(response.cookies)
+        if self.session_cookies:
+            # any session cookies to store
+            cookies_dict.update({k: session.cookies[k] for k in self.session_cookies})
+
+        if cookies_dict:
             if (url in cookies_db) and update:
                 cookies_db[url].update(cookies_dict)
             else:
                 cookies_db[url] = cookies_dict
             # assign cookies for this session
-            session.cookies = response.cookies
+            for c, v in cookies_dict.items():
+                if c not in session.cookies or session.cookies[c] != v:
+                    session.cookies[c] = v  # .update(cookies_dict)
         return response
 
     def _post_credential(self, credentials, post_url, session):
@@ -206,8 +233,11 @@ class HTMLFormAuthenticator(HTTPBaseAuthenticator):
         post_fields = {
             k: v.format(**credentials)
             for k, v in self.fields.items()
-            }
+        }
+
         response = session.post(post_url, data=post_fields)
+        lgr.debug("Posted to %s fields %s, got response %s with headers %s",
+                  post_url, list(post_fields.keys()), response, list(response.headers.keys()))
         return response
 
 
@@ -259,6 +289,81 @@ class HTTPDigestAuthAuthenticator(HTTPRequestsAuthenticator):
         raise NotImplementedError("not yet functioning, see https://github.com/kennethreitz/requests/issues/2934")
 
 
+@auto_repr
+class HTTPDownloaderSession(DownloaderSession):
+    def __init__(self, size=None, filename=None,  url=None, headers=None,
+                 response=None, chunk_size=1024 ** 2):
+        super(HTTPDownloaderSession, self).__init__(
+            size=size, filename=filename, url=url, headers=headers,
+        )
+        self.chunk_size = chunk_size
+        self.response = response
+
+    def download(self, f=None, pbar=None, size=None):
+        response = self.response
+        # content_gzipped = 'gzip' in response.headers.get('content-encoding', '').split(',')
+        # if content_gzipped:
+        #     raise NotImplemented("We do not support (yet) gzipped content")
+        #     # see https://rationalpie.wordpress.com/2010/06/02/python-streaming-gzip-decompression/
+        #     # for ways to implement in python 2 and 3.2's gzip is working better with streams
+
+        total = 0
+        return_content = f is None
+        if f is None:
+            # no file to download to
+            # TODO: actually strange since it should have been decoded then...
+            f = BytesIO()
+
+        # must use .raw to be able avoiding decoding/decompression while downloading
+        # to a file
+        chunk_size_ = min(self.chunk_size, size) if size is not None else self.chunk_size
+
+        # XXX With requests_ftp BytesIO is provided as response.raw for ftp urls,
+        # which has no .stream, so let's do ducktyping and provide our custom stream
+        # via BufferedReader for such cases, while maintaining the rest of code
+        # intact.  TODO: figure it all out, since doesn't scale for any sizeable download
+        if not hasattr(response.raw, 'stream'):
+            def _stream():
+                buf = io.BufferedReader(response.raw)
+                v = True
+                while v:
+                    v = buf.read(chunk_size_)
+                    yield v
+
+            stream = _stream()
+        else:
+            # XXX TODO -- it must be just a dirty workaround
+            # As we discovered with downloads from NITRC all headers come with
+            # Content-Encoding: gzip which leads  requests to decode them.  But the point
+            # is that ftp links (yoh doesn't think) are gzip compressed for the transfer
+            decode_content = not response.url.startswith('ftp://')
+            stream = response.raw.stream(chunk_size_, decode_content=decode_content)
+
+        for chunk in stream:
+            if chunk:  # filter out keep-alive new chunks
+                chunk_len = len(chunk)
+                if size is not None and total + chunk_len > size:
+                    # trim the download to match target size
+                    chunk = chunk[:size - total]
+                    chunk_len = len(chunk)
+                total += chunk_len
+                f.write(chunk)
+                try:
+                    # TODO: pbar is not robust ATM against > 100% performance ;)
+                    if pbar:
+                        pbar.update(total)
+                except Exception as e:
+                    lgr.warning("Failed to update progressbar: %s" % exc_str(e))
+                # TEMP
+                # see https://github.com/niltonvolpato/python-progressbar/pull/44
+                ui.out.flush()
+                if size is not None and total >= size:
+                    break  # we have done as much as we were asked
+
+        if return_content:
+            out = f.getvalue()
+            return out
+
 
 @auto_repr
 class HTTPDownloader(BaseDownloader):
@@ -289,10 +394,10 @@ class HTTPDownloader(BaseDownloader):
                 lgr.debug("http session: Reusing previous")
                 return True  # we used old
             elif url in cookies_db:
-                lgr.debug("http session: Creating new with old cookies")
+                cookie_dict = cookies_db[url]
+                lgr.debug("http session: Creating new with old cookies %s", list(cookie_dict.keys()))
                 self._session = requests.Session()
                 # not sure what happens if cookie is expired (need check to that or exception will prolly get thrown)
-                cookie_dict = cookies_db[url]
 
                 # TODO dict_to_cookiejar doesn't preserve all fields when reversed
                 self._session.cookies = requests.utils.cookiejar_from_dict(cookie_dict)
@@ -308,55 +413,35 @@ class HTTPDownloader(BaseDownloader):
 
         return False
 
-    def _get_download_details(self, url, chunk_size=1024**2, allow_redirects=True):
+    def get_downloader_session(self, url,
+                               allow_redirects=True,
+                               use_redirected_url=True):
         # TODO: possibly make chunk size adaptive
-        response = self._session.get(url, stream=True, allow_redirects=allow_redirects)
+        # TODO: make it not this ugly -- but at the moment we are testing end-file size
+        # while can't know for sure if content was gunziped and either it all went ok.
+        # So safer option -- just request to not have it gzipped
+        headers = {'Accept-Encoding': ''}
+        response = self._session.get(url, stream=True, allow_redirects=allow_redirects, headers=headers)
         check_response_status(response, session=self._session)
         headers = response.headers
+        lgr.debug("Establishing session for url %s, response headers: %s", url, headers)
         target_size = int(headers.get('Content-Length', '0').strip()) or None
+        if use_redirected_url and response.url and response.url != url:
+            lgr.debug("URL %s was redirected to %s and thus the later will be used"
+                      % (url, response.url))
+            url = response.url
         # Consult about filename.  Since we already have headers,
         # should not result in an additional request
         url_filename = get_url_filename(url, headers=headers)
 
-        def _downloader(f=None, pbar=None, size=None):
-            total = 0
-            return_content = f is None
-            if f is None:
-                # no file to download to
-                # TODO: actually strange since it should have been decoded then...
-                f = BytesIO()
-
-            # must use .raw to be able avoiding decoding/decompression while downloading
-            # to a file
-            chunk_size_ = min(chunk_size, size) if size is not None else chunk_size
-            for chunk in response.raw.stream(chunk_size_, decode_content=return_content):
-                if chunk:  # filter out keep-alive new chunks
-                    chunk_len = len(chunk)
-                    if size is not None and total + chunk_len > size:
-                        # trim the download to match target size
-                        chunk = chunk[:size - total]
-                        chunk_len = len(chunk)
-                    total += chunk_len
-                    f.write(chunk)
-                    try:
-                        # TODO: pbar is not robust ATM against > 100% performance ;)
-                        if pbar:
-                            pbar.update(total)
-                    except Exception as e:
-                        lgr.warning("Failed to update progressbar: %s" % exc_str(e))
-                    # TEMP
-                    # see https://github.com/niltonvolpato/python-progressbar/pull/44
-                    ui.out.flush()
-                    if size is not None and total >= size:
-                        break  # we have done as much as we were asked
-            if return_content:
-                out = f.getvalue()
-                if PY3 and isinstance(out, bytes):
-                    out = out.decode()
-                return out
-
-        return _downloader, target_size, url_filename, headers
-
+        headers['Url-Filename'] = url_filename
+        return HTTPDownloaderSession(
+            size=target_size,
+            url=response.url,
+            filename=url_filename,
+            headers=headers,
+            response=response
+        )
 
     @classmethod
     def get_status_from_headers(cls, headers):
@@ -368,7 +453,8 @@ class HTTPDownloader(BaseDownloader):
         HTTP_HEADERS_TO_STATUS = {
             'Content-Length': int,
             'Content-Disposition': str,
-            'Last-Modified': rfc2822_to_epoch
+            'Last-Modified': rfc2822_to_epoch,
+            'Url-Filename': str,
         }
         # Allow for webserver to return them in other casing
         HTTP_HEADERS_TO_STATUS_lower = {s.lower(): (s, t) for s, t in HTTP_HEADERS_TO_STATUS.items()}
@@ -385,5 +471,6 @@ class HTTPDownloader(BaseDownloader):
         return FileStatus(
             size=status.get('Content-Length'),
             mtime=status.get('Last-Modified'),
-            filename=get_response_disposition_filename(status.get('Content-Disposition'))
+            filename=get_response_disposition_filename(
+                status.get('Content-Disposition')) or status.get('Url-Filename')
         )
