@@ -23,19 +23,34 @@ _group_dataset = (
         # src module can be relative, but has to be relative to the main 'datalad' package
         ('datalad.distribution.create', 'Create'),
         ('datalad.distribution.install', 'Install'),
+        ('datalad.distribution.get', 'Get'),
+        ('datalad.distribution.add', 'Add'),
         ('datalad.distribution.publish', 'Publish'),
         ('datalad.distribution.uninstall', 'Uninstall'),
         # N/I ATM
         # ('datalad.distribution.move', 'Move'),
         ('datalad.distribution.update', 'Update'),
-        ('datalad.distribution.create_publication_target_sshwebserver',
-         'CreatePublicationTargetSSHWebserver',
-         'create-publication-target-sshwebserver'),
+        ('datalad.distribution.create_sibling',
+         'CreateSibling',
+         'create-sibling'),
+        ('datalad.distribution.create_sibling_github',
+         'CreateSiblingGithub',
+         'create-sibling-github'),
         ('datalad.distribution.add_sibling', 'AddSibling', 'add-sibling'),
-        ('datalad.distribution.modify_subdataset_urls', 'ModifySubdatasetURLs',
-         'modify-subdataset-urls'),
+        ('datalad.distribution.rewrite_urls', 'RewriteURLs',
+         'rewrite-urls'),
         ('datalad.interface.unlock', 'Unlock', 'unlock'),
         ('datalad.interface.save', 'Save', 'save'),
+        ('datalad.export', 'Export', 'export'),
+    ])
+
+_group_metadata = (
+    'Commands for meta data handling',
+    [
+        ('datalad.metadata.search', 'Search',
+         'search', 'search'),
+        ('datalad.metadata.aggregate', 'AggregateMetaData',
+         'aggregate-metadata', 'aggregate_metadata'),
     ])
 
 _group_misc = (
@@ -43,12 +58,13 @@ _group_misc = (
     [
         ('datalad.interface.test', 'Test'),
         ('datalad.interface.crawl', 'Crawl'),
+        ('datalad.interface.crawl_init', 'CrawlInit', 'crawl-init'),
         ('datalad.interface.ls', 'Ls'),
         ('datalad.interface.clean', 'Clean'),
         ('datalad.interface.add_archive_content', 'AddArchiveContent',
          'add-archive-content'),
         ('datalad.interface.download_url', 'DownloadURL', 'download-url'),
         # very optional ones
-        ('datalad.distribution.tests.create_test_dataset', 'CreateTestDataset',
+        ('datalad.distribution.create_test_dataset', 'CreateTestDataset',
          'create-test-dataset'),
     ])
