@@ -14,9 +14,10 @@ __docformat__ = 'restructuredtext'
 
 import sys
 import re
+import textwrap
 
 from ..ui import ui
-
+from ..dochelpers import exc_str
 
 def get_api_name(intfspec):
     """Given an interface specification return an API name for it"""
@@ -52,7 +53,6 @@ def get_interface_groups():
 
 
 def dedent_docstring(text):
-    import textwrap
     """Remove uniform indentation from a multiline docstring"""
     # Problem is that first line might often have no offset, so might
     # need to be ignored from dedent call
@@ -78,16 +78,31 @@ def alter_interface_docs_for_api(docs):
     docs = dedent_docstring(docs)
     # clean cmdline sections
     docs = re.sub(
-        '\|\| Command line use only \>\>.*\<\< Command line use only \|\|',
+        '\|\| CMDLINE \>\>.*\<\< CMDLINE \|\|',
         '',
         docs,
         flags=re.MULTILINE | re.DOTALL)
     # clean cmdline in-line bits
     docs = re.sub(
-        '\[CMD:\s.*CMD\]',
+        '\[CMD:\s[^\[\]]*\sCMD\]',
         '',
         docs,
         flags=re.MULTILINE | re.DOTALL)
+    docs = re.sub(
+        '\[PY:\s([^\[\]]*)\sPY\]',
+        lambda match: match.group(1),
+        docs,
+        flags=re.MULTILINE)
+    docs = re.sub(
+        '\|\| PYTHON \>\>(.*)\<\< PYTHON \|\|',
+        lambda match: match.group(1),
+        docs,
+        flags=re.MULTILINE | re.DOTALL)
+    docs = re.sub(
+        '\|\| REFLOW \>\>\n(.*)\<\< REFLOW \|\|',
+        lambda match: textwrap.fill(match.group(1)),
+        docs,
+        flags=re.MULTILINE | re.DOTALL)
     return docs
 
 
@@ -100,16 +115,26 @@ def alter_interface_docs_for_cmdline(docs):
     docs = dedent_docstring(docs)
     # clean cmdline sections
     docs = re.sub(
-        '\|\| Python use only \>\>.*\<\< Python use only \|\|',
+        '\|\| PYTHON \>\>.*\<\< PYTHON \|\|',
         '',
         docs,
         flags=re.MULTILINE | re.DOTALL)
     # clean cmdline in-line bits
     docs = re.sub(
-        '\[PY:\s.*PY\]',
+        '\[PY:\s[^\[\]]*\sPY\]',
         '',
         docs,
         flags=re.MULTILINE | re.DOTALL)
+    docs = re.sub(
+        '\[CMD:\s([^\[\]]*)\sCMD\]',
+        lambda match: match.group(1),
+        docs,
+        flags=re.MULTILINE)
+    docs = re.sub(
+        '\|\| CMDLINE \>\>(.*)\<\< CMDLINE \|\|',
+        lambda match: match.group(1),
+        docs,
+        flags=re.MULTILINE | re.DOTALL)
     # remove :role:`...` RST markup for cmdline docs
     docs = re.sub(
         r':\S+:`[^`]*`[\\]*',
@@ -130,9 +155,32 @@ def alter_interface_docs_for_cmdline(docs):
         '`\S*`',
         lambda match: match.group(0).strip('`').upper(),
         docs)
+    # clean up sphinx API refs
+    docs = re.sub(
+        '\~datalad\.api\.\S*',
+        lambda match: "`{0}`".format(match.group(0)[13:]),
+        docs)
+    # Remove RST paragraph markup
+    docs = re.sub(
+        r'^.. \S+::',
+        lambda match: match.group(0)[3:-2].upper(),
+        docs,
+        flags=re.MULTILINE)
+    docs = re.sub(
+        '\|\| REFLOW \>\>\n(.*)\<\< REFLOW \|\|',
+        lambda match: textwrap.fill(match.group(1)),
+        docs,
+        flags=re.MULTILINE | re.DOTALL)
     return docs
 
 
+def is_api_arg(arg):
+    """Return True if argument is our API argument or self or used for internal
+    purposes
+    """
+    return arg != 'self' and not arg.startswith('_')
+
+
 def update_docstring_with_parameters(func, params, prefix=None, suffix=None):
     """Generate a useful docstring from a parameter spec
 
@@ -153,7 +201,7 @@ def update_docstring_with_parameters(func, params, prefix=None, suffix=None):
             doc += '\n'
         doc += "Parameters\n----------\n"
         for i, arg in enumerate(args):
-            if arg == 'self':
+            if not is_api_arg(arg):
                 continue
             # we need a parameter spec for each argument
             if not arg in params:
@@ -165,10 +213,13 @@ def update_docstring_with_parameters(func, params, prefix=None, suffix=None):
             if defaults_idx >= 0:
                 if not param.constraints is None:
                     param.constraints(defaults[defaults_idx])
+            orig_docs = param._doc
+            param._doc = alter_interface_docs_for_api(param._doc)
             doc += param.get_autodoc(
                 arg,
                 default=defaults[defaults_idx] if defaults_idx >= 0 else None,
                 has_default=defaults_idx >= 0)
+            param._doc = orig_docs
             doc += '\n'
     doc += suffix if suffix else u""
     # assign the amended docs
@@ -191,7 +242,7 @@ class Interface(object):
         if not defaults is None:
             ndefaults = len(defaults)
         for i, arg in enumerate(args):
-            if arg == 'self':
+            if not is_api_arg(arg):
                 continue
             param = cls._params_[arg]
             defaults_idx = ndefaults - len(args) + i
@@ -217,7 +268,7 @@ class Interface(object):
                 help += '.'
             if param.constraints is not None:
                 parser_kwargs['type'] = param.constraints
-                # include value contraint description and default
+                # include value constraint description and default
                 # into the help string
                 cdoc = alter_interface_docs_for_cmdline(
                     param.constraints.long_description())
@@ -235,10 +286,21 @@ class Interface(object):
     def call_from_parser(cls, args):
         # XXX needs safety check for name collisions
         from inspect import getargspec
-        argnames = getargspec(cls.__call__)[0]
-        kwargs = {k: getattr(args, k) for k in argnames if k != 'self'}
+        argspec = getargspec(cls.__call__)
+        if argspec[2] is None:
+            # no **kwargs in the call receiver, pull argnames from signature
+            argnames = getargspec(cls.__call__)[0]
+        else:
+            # common options
+            # XXX define or better get from elsewhere
+            common_opts = ('change_path', 'common_debug', 'common_idebug', 'func',
+                           'help', 'log_level', 'logger', 'pbs_runner',
+                           'result_renderer', 'subparser')
+            argnames = [name for name in dir(args)
+                        if not (name.startswith('_') or name in common_opts)]
+        kwargs = {k: getattr(args, k) for k in argnames if is_api_arg(k)}
         try:
             return cls.__call__(**kwargs)
-        except KeyboardInterrupt:
-            ui.error("\nInterrupted by user while doing magic")
+        except KeyboardInterrupt as exc:
+            ui.error("\nInterrupted by user while doing magic: %s" % exc_str(exc))
             sys.exit(1)
