@@ -13,18 +13,20 @@
 from distutils.version import StrictVersion
 import hashlib
 import patoolib
-import re
 # There were issues, so let's stay consistently with recent version
 assert(StrictVersion(patoolib.__version__) >= "1.7")
 
 import os
 import tempfile
-from os.path import join as opj, exists, abspath, basename, isabs, normpath, relpath, pardir, isdir
-from os.path import split as ops, sep as opsep
+from os.path import join as opj, exists, abspath, isabs, normpath, relpath, pardir, isdir
+from os.path import sep as opsep
+from os.path import realpath
 from six import next
 from six.moves.urllib.parse import unquote as urlunquote
 
-from ..utils import getpwd
+import string
+import random
+
 from ..utils import any_re_search
 
 import logging
@@ -58,11 +60,14 @@ from ..utils import swallow_outputs
 from ..utils import rmtemp
 from ..cmd import Runner
 from ..consts import ARCHIVES_TEMP_DIR
-from ..utils import rotree, rmtree
+from ..utils import rmtree
 from ..utils import get_tempfile_kwargs
 
+from ..utils import on_windows
+
 _runner = Runner()
 
+
 def _patool_run(cmd, verbosity=0, **kwargs):
     """Decorated runner for patool so it doesn't spit out outputs to stdout"""
     # use our runner
@@ -86,9 +91,9 @@ DECOMPRESSORS = {
     '\.(tar\.xz)$': 'tar -xJvf %(file)s -C %(dir)s',
     '\.(tar\.gz|tgz)$': 'tar -xzvf %(file)s -C %(dir)s',
     '\.(zip)$': 'unzip %(file)s -d %(dir)s',
-    }
+}
+
 
-from ..utils import on_windows
 def unixify_path(path):
     """On windows convert paths from drive:\d\file to /drive/d/file
 
@@ -202,10 +207,11 @@ def _get_cached_filename(archive):
     """
     #return "%s_%s" % (basename(archive), hashlib.md5(archive).hexdigest()[:5])
     # per se there is no reason to maintain any long original name here.
-    return hashlib.md5(archive.encode()).hexdigest()[:10]
+    archive_cached = hashlib.md5(realpath(archive).encode()).hexdigest()[:10]
+    lgr.debug("Cached directory for archive %s is %s", archive, archive_cached)
+    return archive_cached
+
 
-import string
-import random
 def _get_random_id(size=6, chars=string.ascii_uppercase + string.digits):
     """Return a random ID composed from digits and uppercase letters
 
@@ -213,6 +219,7 @@ def _get_random_id(size=6, chars=string.ascii_uppercase + string.digits):
     """
     return ''.join(random.choice(chars) for _ in range(size))
 
+
 class ArchivesCache(object):
     """Cache to maintain extracted archives
 
@@ -235,7 +242,7 @@ class ArchivesCache(object):
             if not persistent:
                 tempsuffix = "-" + _get_random_id()
                 lgr.debug("For non-persistent archives using %s suffix for path %s",
-                          tempsuffix, path )
+                          tempsuffix, path)
                 path += tempsuffix
         else:
             if persistent:
@@ -262,7 +269,6 @@ class ArchivesCache(object):
             lgr.debug("Not initiating existing cache for the archives under %s" % self.path)
             self._made_path = False
 
-
     @property
     def path(self):
         return self._path
@@ -276,8 +282,8 @@ class ArchivesCache(object):
         #     lgr.debug("Removing the entire archives cache under %s" % self.path)
         #     rmtemp(self.path)
         if (not self.persistent) or force:
-             lgr.debug("Removing the entire archives cache under %s" % self.path)
-             rmtemp(self.path)
+            lgr.debug("Removing the entire archives cache under %s" % self.path)
+            rmtemp(self.path)
 
     def _get_normalized_archive_path(self, archive):
         """Return full path to archive
@@ -321,13 +327,17 @@ class ArchivesCache(object):
             # we can at least try
             if not self.persistent:
                 self.clean()
-        except:
+        except:  # MIH: IOError?
             pass
 
 
 class ExtractedArchive(object):
     """Container for the extracted archive
     """
+
+    # suffix to use for a stamp so we could guarantee that extracted archive is
+    STAMP_SUFFIX = '.stamp'
+
     def __init__(self, archive, path=None, persistent=False):
         self._archive = archive
         # TODO: bad location for extracted archive -- use tempfile
@@ -345,16 +355,21 @@ class ExtractedArchive(object):
 
     def clean(self, force=False):
         # would interfere with tests
-        # if os.environ.get('DATALAD_TESTS_KEEPTEMP'):
+        # if os.environ.get('DATALAD_TESTS_TEMP_KEEP'):
         #     lgr.info("As instructed, not cleaning up the cache under %s"
         #              % self._path)
         #     return
-        if exists(self._path):
-            if (not self._persistent) or force:
-                lgr.debug("Cleaning up the cache for %s under %s", self._archive, self._path)
-                # TODO:  we must be careful here -- to not modify permissions of files
-                #        only of directories
-                rmtree(self._path)
+
+        for path, name in [
+            (self._path, 'cache'),
+            (self.stamp_path, 'stamp file')
+        ]:
+            if exists(path):
+                if (not self._persistent) or force:
+                    lgr.debug("Cleaning up the %s for %s under %s", name, self._archive, path)
+                    # TODO:  we must be careful here -- to not modify permissions of files
+                    #        only of directories
+                    (rmtree if isdir(path) else os.unlink)(path)
 
     @property
     def path(self):
@@ -362,24 +377,48 @@ class ExtractedArchive(object):
         """
         return self._path
 
+    @property
+    def stamp_path(self):
+        return self._path + self.STAMP_SUFFIX
+
+    @property
+    def is_extracted(self):
+        return exists(self.path) and exists(self.stamp_path) \
+            and os.stat(self.stamp_path).st_mtime >= os.stat(self.path).st_mtime
+
     def assure_extracted(self):
         """Return path to the extracted `archive`.  Extract archive if necessary
         """
         path = self.path
-        if not exists(path):
+
+        if not self.is_extracted:
             # we need to extract the archive
             # TODO: extract to _tmp and then move in a single command so we
             # don't end up picking up broken pieces
             lgr.debug("Extracting {self._archive} under {path}".format(**locals()))
+            if exists(path):
+                lgr.debug("Previous extracted (but probably not fully) cached archive found. Removing %s", path)
+                rmtree(path)
+
             os.makedirs(path)
             assert(exists(path))
-
+            # remove old stamp
+            if exists(self.stamp_path):
+                rmtree(self.stamp_path)
             decompress_file(self._archive, path, leading_directories=None)
 
             # TODO: must optional since we might to use this content, move it into the tree etc
             # lgr.debug("Adjusting permissions to R/O for the extracted content")
             # rotree(path)
             assert(exists(path))
+
+            # create a stamp
+            with open(self.stamp_path, 'w') as f:
+                f.write(self._archive)
+
+            # assert that stamp mtime is not older than archive's directory
+            assert(self.is_extracted)
+
         return path
 
     # TODO: remove?
@@ -467,7 +506,5 @@ class ExtractedArchive(object):
         try:
             if self._persistent:
                 self.clean()
-        except:
+        except:  # MIH: IOError?
             pass
-
-
