@@ -7,22 +7,24 @@
 #
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 
+import logging
+lgr = logging.getLogger('datalad.network')
+
+lgr.log(5, "Importing support.network")
 import calendar
 import email.utils
-import gzip
 import os
 import re
-import shutil
-import time
+import iso8601
 
 from collections import OrderedDict
 from os.path import abspath, isabs
 
 from six import string_types
 from six import iteritems
-from six.moves.urllib.request import urlopen, Request
+from six.moves.urllib.request import Request
 from six.moves.urllib.parse import quote as urlquote, unquote as urlunquote
-from six.moves.urllib.parse import urljoin, urlparse, urlsplit, urlunsplit, urlunparse, ParseResult
+from six.moves.urllib.parse import urljoin, urlparse, urlsplit, urlunparse, ParseResult
 from six.moves.urllib.parse import parse_qsl
 from six.moves.urllib.parse import urlencode
 from six.moves.urllib.error import URLError
@@ -30,12 +32,9 @@ from six.moves.urllib.error import URLError
 from datalad.utils import on_windows
 from datalad.consts import DATASETS_TOPURL
 
-from datalad.utils import auto_repr
 # TODO not sure what needs to use `six` here yet
-import requests
-
-import logging
-lgr = logging.getLogger('datalad.network')
+# !!! Lazily import requests where needed -- needs 30ms or so
+# import requests
 
 
 def get_response_disposition_filename(s):
@@ -45,9 +44,12 @@ def get_response_disposition_filename(s):
     if not s:
         return None
     # If the response has Content-Disposition, try to get filename from it
-    cd = dict(map(
-        lambda x: x.strip().split('=') if '=' in x else (x.strip(),''),
-        s.split(';')))
+    cd = map(
+        lambda x: x.strip().split('=', 1) if '=' in x else [x.strip(), ''],
+        s.split(';')
+    )
+    # unify the key to be lower case and make it into a dict
+    cd = dict([[x[0].lower()] + x[1:] for x in cd])
     if 'filename' in cd:
         filename = cd['filename'].strip("\"'")
         return filename
@@ -74,7 +76,7 @@ def get_url_disposition_filename(url, headers=None):
             r.close()
 
 
-def get_url_straight_filename(url, strip=[], allowdir=False):
+def get_url_straight_filename(url, strip=None, allowdir=False):
     """Get file/dir name of the last path component of the URL
 
     Parameters
@@ -105,13 +107,13 @@ def get_url_straight_filename(url, strip=[], allowdir=False):
         return None
 
 
-def get_url_filename(url, headers=None, strip=[]):
+def get_url_filename(url, headers=None, strip=None):
     """Get filename from the url, first consulting server about Content-Disposition
     """
     filename = get_url_disposition_filename(url, headers)
     if filename:
         return filename
-    return get_url_straight_filename(url, strip=[])
+    return get_url_straight_filename(url, strip=strip)
 
 
 def get_url_response_stamp(url, response_info):
@@ -145,22 +147,28 @@ def get_tld(url):
 
 
 from email.utils import parsedate_tz, mktime_tz
+
+
 def rfc2822_to_epoch(datestr):
     """Given rfc2822 date/time format, return seconds since epoch"""
     return mktime_tz(parsedate_tz(datestr))
 
 
-import calendar
-from datetime import datetime
 def iso8601_to_epoch(datestr):
-    return calendar.timegm(datetime.strptime(datestr, "%Y-%m-%dT%H:%M:%S.%fZ").timetuple())
+    """Given ISO 8601 date/time format, return in seconds since epoch
+
+    iso8601 is used to parse properly the time zone information, which
+    can't be parsed with standard datetime strptime
+    """
+    return calendar.timegm(iso8601.parse_date(datestr).timetuple())
 
 
-def __urlopen_requests(url, header_vals=None):
+def __urlopen_requests(url):
     # XXX Workaround for now for ... broken code
     if isinstance(url, Request):
         url = url.get_full_url()
-    return requests.Session().get(url)
+    from requests import Session
+    return Session().get(url)
 
 
 def retry_urlopen(url, retries=3):
@@ -181,6 +189,7 @@ def is_url_quoted(url):
         url_ = urlunquote(url)
         return url != url_
     except:  # problem with unquoting -- then it must be wasn't quoted (correctly)
+        # MIH: ValueError?
         return False
 
 
@@ -348,6 +357,10 @@ class RI(object):
             # RI class was used as a factory
             cls = _guess_ri_cls(ri)
 
+        if cls is RI:
+            # should we fail or just pretend we are nothing??? ;-) XXX
+            raise ValueError("Could not deduce RI type for %r" % (ri,))
+
         ri_obj = super(RI, cls).__new__(cls)
         # Store internally original str
         ri_obj._str = ri
@@ -483,6 +496,13 @@ class RI(object):
         else:
             return self._fields[item]
 
+    def __setattr__(self, item, value):
+        if item.startswith('_') or item not in self._FIELDS:
+            super(RI, self).__setattr__(item, value)
+        else:
+            self._fields[item] = value
+            self._str = None
+
 
 class URL(RI):
     """Universal resource locator
@@ -614,13 +634,12 @@ class URL(RI):
             raise ValueError(
                 "Non 'file://' URL cannot be resolved to a local path")
         hostname = self.hostname
-        if not (hostname in (None, '', 'localhost', '::1') \
+        if not (hostname in (None, '', 'localhost', '::1')
                 or hostname.startswith('127.')):
             raise ValueError("file:// URL does not point to 'localhost'")
         return self.path
 
 
-
 class PathRI(RI):
     """RI pointing to a (local) file/directory"""
     def as_str(self):
@@ -724,7 +743,7 @@ class DataLadRI(RI, RegexBasedURLMixin):
         """
         if self.remote:
             raise NotImplementedError("not supported ATM to reference additional remotes")
-        return "{}{}".format(DATASETS_TOPURL, self.path)
+        return "{}{}".format(DATASETS_TOPURL, urlquote(self.path))
 
 
 def _split_colon(s, maxsplit=1):
@@ -778,7 +797,7 @@ def is_url(ri):
     if not isinstance(ri, RI):
         try:
             ri = RI(ri)
-        except:
+        except:  # MIH: MemoryError?
             return False
     return isinstance(ri, (URL, SSHRI))
 
@@ -792,7 +811,7 @@ def is_datalad_compat_ri(ri):
     if not isinstance(ri, RI):
         try:
             ri = RI(ri)
-        except:
+        except:  # MIH: MemoryError?
             return False
     return isinstance(ri, (URL, SSHRI, DataLadRI))
 
@@ -814,8 +833,8 @@ def is_ssh(ri):
     # string or RI only, but with everything RI itself can deal with:
     _ri = RI(ri) if not isinstance(ri, RI) else ri
 
-    return isinstance(_ri, SSHRI) or \
-           (isinstance(_ri, URL) and _ri.scheme == 'ssh')
+    return isinstance(_ri, SSHRI) \
+        or (isinstance(_ri, URL) and _ri.scheme == 'ssh')
 
 
 #### windows workaround ###
@@ -836,4 +855,6 @@ def get_local_file_url(fname):
     else:
         # TODO:  need to fix for all the encoding etc
         furl = str(URL(scheme='file', path=fname))
-    return furl
\ No newline at end of file
+    return furl
+
+lgr.log(5, "Done importing support.network")
