@@ -23,19 +23,22 @@ import logging
 import datalad.log  # Just to have lgr setup happen this one used a script
 lgr = logging.getLogger('datalad.s3')
 
-from .keyring_ import keyring
-
 from ..dochelpers import exc_str
+from .exceptions import DownloadError, AccessDeniedError
+
+from six.moves.urllib.request import urlopen, Request
+from six.moves.urllib.parse import urlparse, urlunparse
+
 
 try:
     import boto
     from boto.s3.key import Key
     from boto.exception import S3ResponseError
-except ImportError:
-    boto = Key = S3ResponseError = None
+    from boto.s3.connection import OrdinaryCallingFormat
 except Exception as e:
-    lgr.warning("boto module failed to import although available: %s" % exc_str(e))
-    boto = Key = S3ResponseError = None
+    if not isinstance(e, ImportError):
+        lgr.warning("boto module failed to import although available: %s" % exc_str(e))
+    boto = Key = S3ResponseError = OrdinaryCallingFormat = None
 
 
 # TODO: should become a config option and managed along with the rest
@@ -48,14 +51,54 @@ def _get_bucket_connection(credential):
     # with different resources. Thus for now just making an option which
     # one to use
     # do full shebang with entering credentials
-    from ..downloaders.credentials import Credential
-    credential = Credential(credential, "aws-s3", None)
+    from ..downloaders.credentials import AWS_S3
+    credential = AWS_S3(credential, None)
     if not credential.is_known:
         credential.enter_new()
     creds = credential()
     return boto.connect_s3(creds["key_id"], creds["secret_id"])
 
 
+def _handle_exception(e, bucket_name):
+    """Helper to handle S3 connection exception"""
+    if e.error_code == 'AccessDenied':
+        raise AccessDeniedError(exc_str(e))
+    else:
+        raise DownloadError(
+            "Cannot connect to %s S3 bucket. Exception: %s"
+            % (bucket_name, exc_str(e))
+        )
+
+
+def get_bucket(conn, bucket_name):
+    """A helper to get a bucket
+
+    Parameters
+    ----------
+    bucket_name: str
+        Name of the bucket to connect to
+    """
+    try:
+        bucket = conn.get_bucket(bucket_name)
+    except S3ResponseError as e:
+        # can initially deny or error to connect to the specific bucket by name,
+        # and we would need to list which buckets are available under following
+        # credentials:
+        lgr.debug("Cannot access bucket %s by name: %s", bucket_name, exc_str(e))
+        try:
+            all_buckets = conn.get_all_buckets()
+        except S3ResponseError as e2:
+            lgr.debug("Cannot access all buckets: %s", exc_str(e2))
+            _handle_exception(e, 'any')
+        all_bucket_names = [b.name for b in all_buckets]
+        lgr.debug("Found following buckets %s", ', '.join(all_bucket_names))
+        if bucket_name in all_bucket_names:
+            bucket = all_buckets[all_bucket_names.index(bucket_name)]
+        else:
+            _handle_exception(e, bucket_name)
+    return bucket
+
+
 class VersionedFilesPool(object):
     """Just a helper which would help to create versioned files in the bucket"""
     def __init__(self, bucket):
@@ -74,7 +117,7 @@ class VersionedFilesPool(object):
         k.key = filename
         #k.set_contents_from_filename('/home/yoh/.emacs')
         base, ext = splitext(filename)
-        content_type = None
+        #content_type = None
         mtype = mimetypes.guess_type(filename)[0] or 'application/octet-stream'
         headers = {'Content-Type': mtype}
 
@@ -91,15 +134,20 @@ class VersionedFilesPool(object):
     def reset_version(self, filename):
         self._versions[filename] = 0
 
-def get_key_url(e, schema='http'):
+
+def get_key_url(e, schema='http', versioned=True):
     """Generate an s3:// or http:// url given a key
     """
     if schema == 'http':
-        return "http://{e.bucket.name}.s3.amazonaws.com/{e.name}?versionId={e.version_id}".format(e=e)
+        fmt = "http://{e.bucket.name}.s3.amazonaws.com/{e.name}"
     elif schema == 's3':
-        return "s3://{e.bucket.name}/{e.name}?versionId={e.version_id}".format(e=e)
+        fmt = "s3://{e.bucket.name}/{e.name}"
     else:
         raise ValueError(schema)
+    if versioned:
+        fmt += "?versionId={e.version_id}"
+    return fmt.format(e=e)
+
 
 def prune_and_delete_bucket(bucket):
     """Deletes all the content and then deletes bucket
@@ -111,7 +159,8 @@ def prune_and_delete_bucket(bucket):
     #for key in b.list_versions(''):
     #    b.delete_key(key)
     bucket.delete()
-    lgr.info("Bucket %s was removed"  % bucket.name)
+    lgr.info("Bucket %s was removed" % bucket.name)
+
 
 def set_bucket_public_access_policy(bucket):
     # we need to enable permissions for making content available
@@ -127,6 +176,7 @@ def set_bucket_public_access_policy(bucket):
       ]
     }""" % bucket.name)
 
+
 def gen_test_bucket(bucket_name):
     conn = _get_bucket_connection(S3_ADMIN_CREDENTIAL)
     # assure we have none
@@ -134,7 +184,7 @@ def gen_test_bucket(bucket_name):
         bucket = conn.get_bucket(bucket_name)
         lgr.info("Deleting existing bucket %s" % bucket.name)
         prune_and_delete_bucket(bucket)
-    except:
+    except:  # MIH: MemoryError?
         # so nothing to worry about
         pass
     finally:
@@ -142,6 +192,7 @@ def gen_test_bucket(bucket_name):
 
     return conn.create_bucket(bucket_name)
 
+
 def _gen_bucket_test0(bucket_name="datalad-test0", versioned=True):
 
     bucket = gen_test_bucket(bucket_name)
@@ -163,7 +214,7 @@ def _gen_bucket_test0(bucket_name="datalad-test0", versioned=True):
     files("2versions-nonversioned1.txt_sameprefix")
     for v in range(3):
         files("3versions-allversioned.txt")
-    files("3versions-allversioned.txt_sameprefix") # to test possible problems
+    files("3versions-allversioned.txt_sameprefix")  # to test possible problems
 
     # File which was created and then removed
     bucket.delete_key(files("1version-removed.txt"))
@@ -182,12 +233,15 @@ def _gen_bucket_test0(bucket_name="datalad-test0", versioned=True):
 
     return bucket
 
+
 def gen_bucket_test0_versioned():
     return _gen_bucket_test0('datalad-test0-versioned', versioned=True)
 
+
 def gen_bucket_test0_nonversioned():
     return _gen_bucket_test0('datalad-test0-nonversioned', versioned=False)
 
+
 def gen_bucket_test1_dirs():
     bucket_name = 'datalad-test1-dirs-versioned'
     bucket = gen_test_bucket(bucket_name)
@@ -206,9 +260,6 @@ def gen_bucket_test1_dirs():
     files("d1", load="smth")
 
 
-from six.moves.urllib.request import urlopen, Request
-from six.moves.urllib.parse import urljoin, urlparse, urlsplit, urlunsplit, urlunparse, urlencode
-
 def get_versioned_url(url, guarantee_versioned=False, return_all=False, verify=False,
                       s3conn=None):
     """Given a url return a versioned URL
@@ -249,7 +300,7 @@ def get_versioned_url(url, guarantee_versioned=False, return_all=False, verify=F
         # url is s3.amazonaws.com/bucket/PATH
         s3_bucket, fpath = fpath.split('/', 1)
     elif url_rec.scheme == 's3':
-        s3_bucket = url_rec.netloc # must be
+        s3_bucket = url_rec.netloc  # must be
         # and for now implement magical conversion to URL
         # TODO: wouldn't work if needs special permissions etc
         # actually for now
@@ -265,9 +316,14 @@ def get_versioned_url(url, guarantee_versioned=False, return_all=False, verify=F
             providers = Providers.from_config_files()
             s3url = "s3://%s/" % s3_bucket
             s3provider = providers.get_provider(s3url)
-            bucket = s3provider.authenticator.authenticate(s3_bucket, s3provider.credential)  # s3conn or _get_bucket_connection(S3_TEST_CREDENTIAL)
+            if s3provider.authenticator.bucket is not None and s3provider.authenticator.bucket.name == s3_bucket:
+                # we have established connection before, so let's just reuse
+                bucket = s3provider.authenticator.bucket
+            else:
+                bucket = s3provider.authenticator.authenticate(s3_bucket, s3provider.credential)  # s3conn or _get_bucket_connection(S3_TEST_CREDENTIAL)
         else:
             bucket = s3conn.get_bucket(s3_bucket)
+
         supports_versioning = True  # assume that it does
         try:
             supports_versioning = bucket.get_versioning_status()  # TODO cache
@@ -276,12 +332,11 @@ def get_versioned_url(url, guarantee_versioned=False, return_all=False, verify=F
             supports_versioning = 'maybe'
 
         if supports_versioning:
-
             all_keys = bucket.list_versions(fpath)
             # Filter and sort them so the newest one on top
             all_keys = [x for x in sorted(all_keys, key=lambda x: (x.last_modified, x.is_latest))
-                        if (isinstance(x, Key)    # ignore DeleteMarkers
-                            and (x.name == fpath) # match exact name, not just prefix
+                        if (isinstance(x, Key)     # ignore DeleteMarkers
+                            and (x.name == fpath)  # match exact name, not just prefix
                             )
                         ][::-1]
             # our current assumptions
@@ -290,7 +345,7 @@ def get_versioned_url(url, guarantee_versioned=False, return_all=False, verify=F
             for key in all_keys:
                 version_id = key.version_id
                 query = ((url_rec.query + "&") if url_rec.query else "") \
-                        + "versionId=%s" % version_id
+                    + "versionId=%s" % version_id
                 url_versioned = urlunparse(url_rec._replace(query=query))
                 all_versions.append(url_versioned)
                 if verify:
