@@ -10,13 +10,14 @@
 
 """
 
+from functools import partial
+from os import mkdir
+from shutil import copyfile
+
 from six.moves.urllib.parse import urljoin
 from six.moves.urllib.parse import urlsplit
-from shutil import copyfile
-from nose.tools import assert_is_instance
 
 from datalad.tests.utils import *
-
 # imports from same module:
 from ..annexrepo import *
 
@@ -33,7 +34,7 @@ def test_AnnexRepo_instance_from_clone(src, dst):
 
     # do it again should raise GitCommandError since git will notice
     # there's already a git-repo at that path and therefore can't clone to `dst`
-    with swallow_logs() as cm:
+    with swallow_logs(new_level=logging.WARN) as cm:
         assert_raises(GitCommandError, AnnexRepo, dst, src)
         if git.__version__ != "1.0.2" and git.__version__ != "2.0.5":
             assert("already exists" in cm.out)
@@ -99,6 +100,24 @@ def test_AnnexRepo_is_direct_mode(path):
         assert_false(dm)
 
 
+@with_tempfile()
+def test_AnnexRepo_is_direct_mode_gitrepo(path):
+    repo = GitRepo(path, create=True)
+    # artificially make .git/annex so no annex section gets initialized
+    # in .git/config.  We did manage somehow to make this happen (via publish)
+    # but didn't reproduce yet, so just creating manually
+    mkdir(opj(repo.path, '.git', 'annex'))
+    ar = AnnexRepo(path, init=False, create=False)
+    # It is unlikely though that annex would be in direct mode (requires explicit)
+    # annex magic, without having annex section under .git/config
+    dm = ar.is_direct_mode()
+
+    if ar.is_crippled_fs() or on_windows:
+        assert_true(dm)
+    else:
+        assert_false(dm)
+
+
 @assert_cwd_unchanged
 @with_testrepos('.*annex.*')
 @with_tempfile
@@ -124,6 +143,14 @@ def test_AnnexRepo_annex_proxy(src, annex_path):
     ar.set_direct_mode(True)
     ok_clean_git_annex_proxy(path=annex_path)
 
+    # annex proxy raises in indirect mode:
+    try:
+        ar.set_direct_mode(False)
+        assert_raises(CommandNotAvailableError, ar.proxy, ['git', 'status'])
+    except CommandNotAvailableError:
+        # we can't switch to indirect
+        pass
+
 
 @assert_cwd_unchanged
 @with_testrepos('.*annex.*', flavors=local_testrepo_flavors)
@@ -137,6 +164,13 @@ def test_AnnexRepo_get_file_key(src, annex_path):
         ar.get_file_key("test-annex.dat"),
         'SHA256E-s4--181210f8f9c779c26da1d9b2075bde0127302ee0e3fca38c9a83f5b1dd8e5d3b.dat')
 
+    # and should take a list with an empty string as result, if a file wasn't
+    # in annex:
+    assert_equal(
+        ar.get_file_key(["filenotpresent.wtf", "test-annex.dat"]),
+        ['', 'SHA256E-s4--181210f8f9c779c26da1d9b2075bde0127302ee0e3fca38c9a83f5b1dd8e5d3b.dat']
+    )
+
     # test.dat is actually in git
     # should raise Exception; also test for polymorphism
     assert_raises(IOError, ar.get_file_key, "test.dat")
@@ -147,23 +181,54 @@ def test_AnnexRepo_get_file_key(src, annex_path):
     assert_raises(IOError, ar.get_file_key, "filenotpresent.wtf")
 
 
+
+
+@with_tempfile(mkdir=True)
+def test_AnnexRepo_get_outofspace(annex_path):
+    ar = AnnexRepo(annex_path, create=True)
+
+    def raise_cmderror(*args, **kwargs):
+        raise CommandError(
+            cmd="whatever",
+            stderr="junk around not enough free space, need 905.6 MB more and after"
+        )
+
+    with patch.object(AnnexRepo, '_run_annex_command', raise_cmderror) as cma, \
+        assert_raises(OutOfSpaceError) as cme:
+        ar.get("file")
+    exc = cme.exception
+    assert_equal(exc.sizemore_msg, '905.6 MB')
+    assert_re_in(".*annex (find|get). needs 905.6 MB more", str(exc))
+
+
+@with_testrepos('basic_annex', flavors=['local'])
+def test_AnnexRepo_get_remote_na(path):
+    ar = AnnexRepo(path)
+
+    with assert_raises(RemoteNotAvailableError) as cme:
+        ar.get('test-annex.dat', options=["--from=NotExistingRemote"])
+    eq_(cme.exception.remote, "NotExistingRemote")
+
+
 # 1 is enough to test file_has_content
+@with_batch_direct
 @with_testrepos('.*annex.*', flavors=['local'], count=1)
 @with_tempfile
-def test_AnnexRepo_file_has_content(src, annex_path):
-    ar = AnnexRepo(annex_path, src)
+def test_AnnexRepo_file_has_content(batch, direct, src, annex_path):
+    ar = AnnexRepo(annex_path, src, direct=direct)
     testfiles = ["test-annex.dat", "test.dat"]
+
     assert_equal(ar.file_has_content(testfiles), [False, False])
 
     ok_annex_get(ar, "test-annex.dat")
-    assert_equal(ar.file_has_content(testfiles), [True, False])
-    assert_equal(ar.file_has_content(testfiles[:1]), [True])
+    assert_equal(ar.file_has_content(testfiles, batch=batch), [True, False])
+    assert_equal(ar.file_has_content(testfiles[:1], batch=batch), [True])
 
-    assert_equal(ar.file_has_content(testfiles + ["bogus.txt"]),
+    assert_equal(ar.file_has_content(testfiles + ["bogus.txt"], batch=batch),
                  [True, False, False])
 
-    assert_false(ar.file_has_content("bogus.txt"))
-    assert_true(ar.file_has_content("test-annex.dat"))
+    assert_false(ar.file_has_content("bogus.txt", batch=batch))
+    assert_true(ar.file_has_content("test-annex.dat", batch=batch))
 
 
 # 1 is enough to test
@@ -193,17 +258,6 @@ def test_AnnexRepo_is_under_annex(batch, direct, src, annex_path):
     assert_true(ar.is_under_annex("test-annex.dat", batch=batch))
 
 
-def test_AnnexRepo_options_decorator():
-
-    @kwargs_to_options
-    def decorated(self, whatever, options=[]):
-        return options
-
-    # Order is not guaranteed so use sets
-    assert_equal(set(decorated(1, 2, someoption='first', someotheroption='second')),
-                 {' --someoption=first', ' --someotheroption=second'})
-
-
 @with_tree(tree=(('about.txt', 'Lots of abouts'),
                  ('about2.txt', 'more abouts'),
                  ('d', {'sub.txt': 'more stuff'})))
@@ -241,20 +295,25 @@ def test_AnnexRepo_web_remote(sitepath, siteurl, dst):
     ldesc = ar.whereis(testfile, output='descriptions')
     assert_equal(set(ldesc), set([v['description'] for v in lfull.values()]))
 
-    # info
-    info = ar.info(testfile)
-    assert_equal(info['size'], 14)
-    assert(info['key'])  # that it is there
-    info_batched = ar.info(testfile, batch=True)
-    assert_equal(info, info_batched)
-    # while at it ;)
-    assert_equal(ar.info('nonexistent', batch=False), None)
-    assert_equal(ar.info('nonexistent', batch=True), None)
+    # info w/ and w/o fast mode
+    for fast in [True, False]:
+        info = ar.info(testfile, fast=fast)
+        assert_equal(info['size'], 14)
+        assert(info['key'])  # that it is there
+        info_batched = ar.info(testfile, batch=True, fast=fast)
+        assert_equal(info, info_batched)
+        # while at it ;)
+        assert_equal(ar.info('nonexistent', batch=False), None)
+        assert_equal(ar.info('nonexistent', batch=True), None)
 
     # annex repo info
-    repo_info = ar.repo_info()
+    repo_info = ar.repo_info(fast=False)
     assert_equal(repo_info['local annex size'], 14)
     assert_equal(repo_info['backend usage'], {'SHA256E': 1})
+    # annex repo info in fast mode
+    repo_info_fast = ar.repo_info(fast=True)
+    # doesn't give much testable info, so just comparing a subset for match with repo_info info
+    assert_equal(repo_info_fast['semitrusted repositories'], repo_info['semitrusted repositories'])
     #import pprint; pprint.pprint(repo_info)
 
     # remove the remote
@@ -342,7 +401,6 @@ def test_AnnexRepo_web_remote(sitepath, siteurl, dst):
         assert_equal(info2_[cur_subfile]['size'], 10)
 
 
-
 @with_testrepos('.*annex.*', flavors=['local', 'network'])
 @with_tempfile
 def test_AnnexRepo_migrating_backends(src, dst):
@@ -550,9 +608,7 @@ def test_AnnexRepo_commit(src, path):
     filename = opj(path, get_most_obscure_supported_name())
     with open(filename, 'w') as f:
         f.write("File to add to git")
-    # TODO: Ths wrong now, since add will annex_add in that case
-    # => assertions insufficient!
-    ds.add(filename)
+    ds.add(filename, git=True)
 
     if ds.is_direct_mode():
         assert_raises(AssertionError, ok_clean_git_annex_proxy, path)
@@ -698,17 +754,41 @@ def test_AnnexRepo_add_to_git(path_1, path_2):
 @with_tempfile
 def test_AnnexRepo_get(src, dst):
 
-    ds = AnnexRepo(dst, src)
-    assert_is_instance(ds, AnnexRepo, "AnnexRepo was not created.")
+    annex = AnnexRepo(dst, src)
+    assert_is_instance(annex, AnnexRepo, "AnnexRepo was not created.")
     testfile = 'test-annex.dat'
     testfile_abs = opj(dst, testfile)
-    assert_false(ds.file_has_content("test-annex.dat"))
-    with swallow_outputs() as cmo:
-        ds.get(testfile)
-    assert_true(ds.file_has_content("test-annex.dat"))
-    f = open(testfile_abs, 'r')
-    assert_equal(f.readlines(), ['123\n'],
-                 "test-annex.dat's content doesn't match.")
+    assert_false(annex.file_has_content("test-annex.dat"))
+    with swallow_outputs():
+        annex.get(testfile)
+    assert_true(annex.file_has_content("test-annex.dat"))
+    ok_file_has_content(testfile_abs, '123', strip=True)
+
+    called = []
+    # for some reason yoh failed mock to properly just call original func
+    orig_run = annex._run_annex_command
+
+    def check_run(cmd, annex_options, **kwargs):
+        called.append(cmd)
+        if cmd == 'find':
+            assert_not_in('-J5', annex_options)
+        elif cmd == 'get':
+            assert_in('-J5', annex_options)
+        else:
+            raise AssertionError(
+                "no other commands so far should be ran. Got %s, %s" %
+                (cmd, annex_options)
+            )
+        return orig_run(cmd, annex_options=annex_options, **kwargs)
+
+    annex.drop(testfile)
+    with patch.object(AnnexRepo, '_run_annex_command',
+                      side_effect=check_run, auto_spec=True), \
+            swallow_outputs():
+        annex.get(testfile, jobs=5)
+    assert_equal(called, ['find', 'get'])
+    ok_file_has_content(testfile_abs, '123', strip=True)
+
 
 
 # TODO:
@@ -716,7 +796,8 @@ def test_AnnexRepo_get(src, dst):
 #def enable_remote(self, name):
 
 @with_testrepos('basic_annex$', flavors=['clone'])
-def _test_AnnexRepo_get_contentlocation(batch, path):
+@with_tempfile
+def _test_AnnexRepo_get_contentlocation(batch, path, work_dir_outside):
     annex = AnnexRepo(path, create=False, init=False)
     fname = 'test-annex.dat'
     key = annex.get_file_key(fname)
@@ -731,8 +812,15 @@ def _test_AnnexRepo_get_contentlocation(batch, path):
     eq_(os.path.realpath(opj(annex.path, fname)),
         os.path.realpath(opj(annex.path, key_location)))
 
-    # TODO: test how it would look if done under a subdir
-    with chpwd('subdir', mkdir=True):
+    # test how it would look if done under a subdir of the annex:
+    with chpwd(opj(annex.path, 'subdir'), mkdir=True):
+        key_location = annex.get_contentlocation(key, batch=batch)
+        # they both should point to the same location eventually
+        eq_(os.path.realpath(opj(annex.path, fname)),
+            os.path.realpath(opj(annex.path, key_location)))
+
+    # test how it would look if done under a dir outside of the annex:
+    with chpwd(work_dir_outside, mkdir=True):
         key_location = annex.get_contentlocation(key, batch=batch)
         # they both should point to the same location eventually
         eq_(os.path.realpath(opj(annex.path, fname)),
@@ -928,13 +1016,59 @@ def test_annex_ssh(repo_path, remote_1_path, remote_2_path):
     ok_(exists(socket_2))
 
 
+@with_testrepos('basic_annex', flavors=['clone'])
+@with_tempfile(mkdir=True)
+def test_annex_remove(path1, path2):
+    ar1 = AnnexRepo(path1, create=False)
+    ar2 = AnnexRepo(path2, path1, create=True, direct=True)
+
+    for repo in (ar1, ar2):
+        file_list = repo.get_annexed_files()
+        assert len(file_list) >= 1
+        # remove a single file
+        out = repo.remove(file_list[0])
+        assert_not_in(file_list[0], repo.get_annexed_files())
+        eq_(out[0], file_list[0])
+
+        with open(opj(repo.path, "rm-test.dat"), "w") as f:
+            f.write("whatever")
+
+        # add it
+        repo.add("rm-test.dat")
+
+        # remove without '--force' should fail, due to staged changes:
+        if repo.is_direct_mode():
+            assert_raises(CommandError, repo.remove, "rm-test.dat")
+        else:
+            assert_raises(GitCommandError, repo.remove, "rm-test.dat")
+        assert_in("rm-test.dat", repo.get_annexed_files())
+
+        # now force:
+        out = repo.remove("rm-test.dat", force=True)
+        assert_not_in("rm-test.dat", repo.get_annexed_files())
+        eq_(out[0], "rm-test.dat")
+
+
 @with_tempfile
-def test_repo_version(path):
-    annex = AnnexRepo(path, create=True, version=6)
-    ok_clean_git(path, annex=True)
+@with_tempfile
+@with_tempfile
+def test_repo_version(path1, path2, path3):
+    annex = AnnexRepo(path1, create=True, version=6)
+    ok_clean_git(path1, annex=True)
     version = annex.repo.config_reader().get_value('annex', 'version')
     eq_(version, 6)
 
+    # default from config item (via env var):
+    with patch.dict('os.environ', {'DATALAD_REPO_VERSION': '6'}):
+        annex = AnnexRepo(path2, create=True)
+        version = annex.repo.config_reader().get_value('annex', 'version')
+        eq_(version, 6)
+
+        # parameter `version` still has priority over default config:
+        annex = AnnexRepo(path3, create=True, version=5)
+        version = annex.repo.config_reader().get_value('annex', 'version')
+        eq_(version, 5)
+
 
 @with_testrepos('.*annex.*', flavors=['clone'])
 @with_tempfile(mkdir=True)
@@ -990,6 +1124,7 @@ def test_annex_drop(src, dst):
     # too much arguments:
     assert_raises(CommandError, ar.drop, ['.'], options=['--all'])
 
+
 @with_testrepos('basic_annex', flavors=['clone'])
 @with_tempfile(mkdir=True)
 def test_annex_remove(path1, path2):
@@ -1022,3 +1157,208 @@ def test_annex_remove(path1, path2):
         assert_not_in("rm-test.dat", repo.get_annexed_files())
         eq_(out[0], "rm-test.dat")
 
+
+@with_batch_direct
+@with_testrepos('basic_annex', flavors=['clone'], count=1)
+def test_is_available(batch, direct, p):
+    annex = AnnexRepo(p)
+
+    # bkw = {'batch': batch}
+    if batch:
+        is_available = partial(annex.is_available, batch=batch)
+    else:
+        is_available = annex.is_available
+
+    fname = 'test-annex.dat'
+    key = annex.get_file_key(fname)
+
+    # explicit is to verify data type etc
+    assert is_available(key, key=True) is True
+    assert is_available(fname) is True
+
+    # known remote but doesn't have it
+    assert is_available(fname, remote='origin') is False
+    # it is on the 'web'
+    assert is_available(fname, remote='web') is True
+    # not effective somehow :-/  may be the process already running or smth
+    # with swallow_logs(), swallow_outputs():  # it will complain!
+    assert is_available(fname, remote='unknown') is False
+    assert_false(is_available("boguskey", key=True))
+
+    # remove url
+    urls = annex.get_urls(fname) #, **bkw)
+    assert(len(urls) == 1)
+    annex.rm_url(fname, urls[0])
+
+    assert is_available(key, key=True) is False
+    assert is_available(fname) is False
+    assert is_available(fname, remote='web') is False
+
+
+@with_tempfile(mkdir=True)
+def test_annex_add_no_dotfiles(path):
+    ar = AnnexRepo(path, create=True)
+    print(ar.path)
+    assert_true(os.path.exists(ar.path))
+    assert_false(ar.repo.is_dirty(
+        index=True, working_tree=True, untracked_files=True, submodules=True))
+    os.makedirs(opj(ar.path, '.datalad'))
+    # we don't care about empty directories
+    assert_false(ar.repo.is_dirty(
+        index=True, working_tree=True, untracked_files=True, submodules=True))
+    with open(opj(ar.path, '.datalad', 'somefile'), 'w') as f:
+        f.write('some content')
+    # make sure the repo is considered dirty now
+    assert_true(ar.repo.is_dirty(
+        index=False, working_tree=False, untracked_files=True, submodules=False))
+    # no file is being added, as dotfiles/directories are ignored by default
+    ar.add('.', git=False)
+    # double check, still dirty
+    assert_true(ar.repo.is_dirty(
+        index=False, working_tree=False, untracked_files=True, submodules=False))
+    # now add to git, and it should work
+    ar.add('.', git=True)
+    # all in index
+    assert_false(ar.repo.is_dirty(
+        index=False, working_tree=True, untracked_files=True, submodules=True))
+    ar.commit(msg="some")
+    # all committed
+    assert_false(ar.repo.is_dirty(
+        index=True, working_tree=True, untracked_files=True, submodules=True))
+    # not known to annex
+    assert_false(ar.is_under_annex(opj(ar.path, '.datalad', 'somefile')))
+
+
+@with_tempfile
+def test_annex_version_handling(path):
+    with patch.object(AnnexRepo, 'git_annex_version', None) as cmpov, \
+         patch.object(AnnexRepo, '_check_git_annex_version',
+                      auto_spec=True,
+                      side_effect=AnnexRepo._check_git_annex_version) \
+            as cmpc, \
+         patch.object(external_versions, '_versions',
+                      {'cmd:annex': AnnexRepo.GIT_ANNEX_MIN_VERSION}):
+            eq_(AnnexRepo.git_annex_version, None)
+            ar1 = AnnexRepo(path, create=True)
+            assert(ar1)
+            eq_(AnnexRepo.git_annex_version, AnnexRepo.GIT_ANNEX_MIN_VERSION)
+            eq_(cmpc.call_count, 1)
+            # 2nd time must not be called
+            ar2 = AnnexRepo(path)
+            assert(ar2)
+            eq_(AnnexRepo.git_annex_version, AnnexRepo.GIT_ANNEX_MIN_VERSION)
+            eq_(cmpc.call_count, 1)
+
+    with patch.object(AnnexRepo, 'git_annex_version', None) as cmpov, \
+            patch.object(AnnexRepo, '_check_git_annex_version',
+                         auto_spec=True,
+                         side_effect=AnnexRepo._check_git_annex_version):
+        # no git-annex at all
+        with patch.object(
+                external_versions, '_versions', {'cmd:annex': None}):
+            eq_(AnnexRepo.git_annex_version, None)
+            with assert_raises(MissingExternalDependency) as cme:
+                AnnexRepo(path)
+            if linux_distribution_name == 'debian':
+                assert_in("http://neuro.debian.net", str(cme.exception))
+            eq_(AnnexRepo.git_annex_version, None)
+
+        # outdated git-annex at all
+        with patch.object(
+                external_versions, '_versions', {'cmd:annex': '6.20160505'}):
+            eq_(AnnexRepo.git_annex_version, None)
+            assert_raises(OutdatedExternalDependency, AnnexRepo, path)
+            # and we don't assign it
+            eq_(AnnexRepo.git_annex_version, None)
+            # so we could still fail
+            assert_raises(OutdatedExternalDependency, AnnexRepo, path)
+
+
+def test_ProcessAnnexProgressIndicators():
+    irrelevant_lines = (
+        'abra',
+        '{"some_json": "sure thing"}'
+    )
+    # regular lines, without completion for known downloads
+    success_lines = (
+        '{"command":"get","note":"","success":true,"key":"key1","file":"file1"}',
+        '{"command":"comm","note":"","success":true,"key":"backend-s10--key2"}',
+    )
+    progress_lines = (
+        '{"byte-progress":10,"action":{"command":"get","note":"from web...",'
+            '"key":"key1","file":"file1"},"percent-progress":"10%"}',
+    )
+
+    # without providing expected entries
+    proc = ProcessAnnexProgressIndicators()
+    # when without any target downloads, there is no total_pbar
+    assert_is(proc.total_pbar, None)
+    # for regular lines -- should just return them without side-effects
+    for l in irrelevant_lines + success_lines:
+        with swallow_outputs() as cmo:
+            assert_equal(proc(l), l)
+            assert_equal(proc.pbars, {})
+            assert_equal(cmo.out, '')
+            assert_equal(cmo.err, '')
+    # should process progress lines
+    assert_equal(proc(progress_lines[0]), None)
+    assert_equal(len(proc.pbars), 1)
+    # but when we finish download -- should get cleared
+    assert_equal(proc(success_lines[0]), success_lines[0])
+    assert_equal(proc.pbars, {})
+    # and no side-effect of any kind in finish
+    assert_equal(proc.finish(), None)
+
+    proc = ProcessAnnexProgressIndicators(expected={'key1': 100, 'key2': None})
+    # when without any target downloads, there is no total_pbar
+    assert(proc.total_pbar is not None)
+    assert_equal(proc.total_pbar._pbar.total, 100)  # as much as it knows at this point
+    # for regular lines -- should still just return them without side-effects
+    for l in irrelevant_lines:
+        with swallow_outputs() as cmo:
+            assert_equal(proc(l), l)
+            assert_equal(proc.pbars, {})
+            assert_equal(cmo.out, '')
+            assert_equal(cmo.err, '')
+    # should process progress lines
+    # it doesn't swallow everything -- so there will be side-effects in output
+    with swallow_outputs() as cmo:
+        assert_equal(proc(progress_lines[0]), None)
+        assert_equal(len(proc.pbars), 1)
+        # but when we finish download -- should get cleared
+        assert_equal(proc(success_lines[0]), success_lines[0])
+        assert_equal(proc.pbars, {})
+        out = cmo.out
+    assert out  # just assert that something was output
+    assert proc.total_pbar is not None
+    # and no side-effect of any kind in finish
+    with swallow_outputs() as cmo:
+        assert_equal(proc.finish(), None)
+        assert_equal(proc.total_pbar, None)
+
+
+@with_tempfile
+@with_tempfile
+def test_get_description(path1, path2):
+    annex1 = AnnexRepo(path1, create=True)
+    # some content for git-annex branch
+    create_tree(path1, {'1.dat': 'content'})
+    annex1.add('1.dat', git=False)
+    annex1.commit("msg")
+    annex1_description = annex1.get_description()
+    assert_not_equal(annex1_description, path1)
+
+    annex2 = AnnexRepo(path2, create=True, description='custom 2')
+    assert_equal(annex2.get_description(), 'custom 2')
+    # not yet known
+    assert_equal(annex2.get_description(uuid=annex1.uuid), None)
+
+    annex2.add_remote('annex1', path1)
+    annex2.fetch('annex1')
+    # it will match the remote name
+    assert_equal(annex2.get_description(uuid=annex1.uuid),
+                 annex1_description + ' [annex1]')
+    # but let's remove the remote
+    annex1.merge_annex('annex1')
+    annex2.remove_remote('annex1')
+    assert_equal(annex2.get_description(uuid=annex1.uuid), annex1_description)
