@@ -12,6 +12,7 @@
 
 from nose.tools import assert_is_instance
 
+import os
 from datalad.tests.utils import *
 from datalad.tests.utils_testrepos import BasicAnnexTestRepo
 from datalad.utils import getpwd, chpwd
@@ -23,6 +24,15 @@ from ..gitrepo import _normalize_path
 from ..exceptions import FileNotInRepositoryError
 
 
+@with_tempfile(mkdir=True)
+def test_GitRepo_invalid_path(path):
+    with chpwd(path):
+        assert_raises(ValueError, GitRepo, path="git://some/url", create=True)
+        ok_(not exists(opj(path, "git:")))
+        assert_raises(ValueError, GitRepo, path="file://some/relative/path", create=True)
+        ok_(not exists(opj(path, "file:")))
+
+
 @assert_cwd_unchanged
 @with_testrepos(flavors=local_testrepo_flavors)
 @with_tempfile
@@ -76,6 +86,15 @@ def test_GitRepo_instance_from_not_existing(path, path2):
     ok_clean_git(path, annex=False)
 
 
+@with_tempfile
+def test_GitRepo_init_options(path):
+    # passing an option, not explicitly defined in GitRepo class:
+    gr = GitRepo(path, create=True, bare=True)
+
+    cfg = gr.repo.config_reader()
+    ok_(cfg.get_value(section="core", option="bare"))
+
+
 @with_tempfile
 @with_tempfile
 def test_GitRepo_equals(path1, path2):
@@ -98,8 +117,9 @@ def test_GitRepo_add(src, path):
     filename = get_most_obscure_supported_name()
     with open(opj(path, filename), 'w') as f:
         f.write("File to add to git")
-    gr.add(filename)
+    added = gr.add(filename)
 
+    assert_equal(added, {'success': True, 'file': filename})
     assert_in(filename, gr.get_indexed_files(),
               "%s not successfully added to %s" % (filename, path))
     # uncommitted:
@@ -112,7 +132,8 @@ def test_GitRepo_add(src, path):
     assert_raises(AssertionError, gr.add, filename, git=None)
 
     # include committing:
-    gr.add(filename, commit=True, msg="Add two files.")
+    added2 = gr.add(filename, commit=True, msg="Add two files.")
+    assert_equal(added2, {'success': True, 'file': filename})
 
     assert_in(filename, gr.get_indexed_files(),
               "%s not successfully added to %s" % (filename, path))
@@ -140,6 +161,7 @@ def test_GitRepo_remove(path):
 
     eq_(set(gr.remove('*', r=True, f=True)), {'file2', 'd2/f1', 'd2/f2'})
 
+
 @assert_cwd_unchanged
 @with_tempfile
 def test_GitRepo_commit(path):
@@ -153,6 +175,14 @@ def test_GitRepo_commit(path):
     gr.commit("Testing GitRepo.commit().")
     ok_clean_git(path, annex=False, untracked=[])
 
+    with open(opj(path, filename), 'w') as f:
+        f.write("changed content")
+
+    gr.add(filename)
+    gr.commit("commit with options", options=to_options(dry_run=True))
+    # wasn't actually committed:
+    ok_(gr.repo.is_dirty())
+
 
 @with_testrepos(flavors=local_testrepo_flavors)
 @with_tempfile
@@ -163,7 +193,7 @@ def test_GitRepo_get_indexed_files(src, path):
 
     runner = Runner()
     out = runner(['git', 'ls-files'], cwd=path)
-    out_list = out[0].split()
+    out_list = list(filter(bool, out[0].split('\n')))
 
     for item in idx_list:
         assert_in(item, out_list, "%s not found in output of git ls-files in %s" % (item, path))
@@ -373,13 +403,16 @@ def test_GitRepo_fetch(test_path, orig_path, clone_path):
     clone = GitRepo(clone_path, orig_path)
     filename = get_most_obscure_supported_name()
 
-    origin.checkout("new_branch", "-b")
+    origin.checkout("new_branch", ['-b'])
     with open(opj(orig_path, filename), 'w') as f:
         f.write("New file.")
     origin.add(filename)
     origin.commit("new file added.")
 
-    clone.fetch(remote='origin')
+    fetched = clone.fetch(remote='origin')
+    # test FetchInfo list returned by fetch
+    eq_([u'origin/master', u'origin/new_branch'],
+        [commit.name for commit in fetched])
 
     ok_clean_git(clone.path, annex=False)
     assert_in("origin/new_branch", clone.get_remote_branches())
@@ -402,7 +435,8 @@ def test_GitRepo_ssh_fetch(remote_path, repo_path):
     # we don't know any branches of the remote:
     eq_([], repo.get_remote_branches())
 
-    repo.fetch(remote="ssh-remote")
+    fetched = repo.fetch(remote="ssh-remote")
+    assert_in('ssh-remote/master', [commit.name for commit in fetched])
     ok_clean_git(repo.path, annex=False)
 
     # the connection is known to the SSH manager, since fetch() requested it:
@@ -427,7 +461,7 @@ def test_GitRepo_ssh_pull(remote_path, repo_path):
     repo.add_remote("ssh-remote", url)
 
     # modify remote:
-    remote_repo.checkout("ssh-test", "-b")
+    remote_repo.checkout("ssh-test", ['-b'])
     with open(opj(remote_repo.path, "ssh_testfile.dat"), "w") as f:
         f.write("whatever")
     remote_repo.add("ssh_testfile.dat")
@@ -462,7 +496,7 @@ def test_GitRepo_ssh_push(repo_path, remote_path):
     repo.add_remote("ssh-remote", url)
 
     # modify local repo:
-    repo.checkout("ssh-test", "-b")
+    repo.checkout("ssh-test", ['-b'])
     with open(opj(repo.path, "ssh_testfile.dat"), "w") as f:
         f.write("whatever")
     repo.add("ssh_testfile.dat")
@@ -472,7 +506,9 @@ def test_GitRepo_ssh_push(repo_path, remote_path):
     assert_not_in("ssh_testfile.dat", remote_repo.get_indexed_files())
 
     # push changes:
-    repo.push(remote="ssh-remote", refspec="ssh-test")
+    pushed = repo.push(remote="ssh-remote", refspec="ssh-test")
+    # test PushInfo object for
+    assert_in("ssh-remote/ssh-test", [commit.remote_ref.name for commit in pushed])
 
     # the connection is known to the SSH manager, since fetch() requested it:
     assert_in(socket_path, ssh_manager._connections)
@@ -519,7 +555,7 @@ def test_GitRepo_remote_update(path1, path2, path3):
         f.write("git2 in master")
     git2.add('masterfile')
     git2.commit("Add something to master.")
-    git2.checkout('branch2', '-b')
+    git2.checkout('branch2', ['-b'])
     with open(opj(path2, 'branch2file'), 'w') as f:
         f.write("git2 in branch2")
     git2.add('branch2file')
@@ -530,7 +566,7 @@ def test_GitRepo_remote_update(path1, path2, path3):
         f.write("git3 in master")
     git3.add('masterfile')
     git3.commit("Add something to master.")
-    git3.checkout('branch3', '-b')
+    git3.checkout('branch3', ['-b'])
     with open(opj(path3, 'branch3file'), 'w') as f:
         f.write("git3 in branch3")
     git3.add('branch3file')
@@ -572,7 +608,7 @@ def test_GitRepo_get_files(url, path):
     eq_(local_files, os_files)
 
     # create a different branch:
-    gr.checkout('new_branch', '-b')
+    gr.checkout('new_branch', ['-b'])
     filename = 'another_file.dat'
     with open(opj(path, filename), 'w') as f:
         f.write("something")
@@ -598,14 +634,17 @@ def test_GitRepo_get_files(url, path):
 @with_tempfile(mkdir=True)
 def test_GitRepo_get_toppath(repo, tempdir):
     reporeal = realpath(repo)
-    eq_(GitRepo.get_toppath(repo), reporeal)
+    eq_(GitRepo.get_toppath(repo, follow_up=False), reporeal)
+    eq_(GitRepo.get_toppath(repo), repo)
     # Generate some nested directory
     nested = opj(repo, "d1", "d2")
     os.makedirs(nested)
-    eq_(GitRepo.get_toppath(nested), reporeal)
+    eq_(GitRepo.get_toppath(nested, follow_up=False), reporeal)
+    eq_(GitRepo.get_toppath(nested), repo)
     # and if not under git, should return None
     eq_(GitRepo.get_toppath(tempdir), None)
 
+
 def test_GitRepo_dirty():
     trepo = BasicAnnexTestRepo()
     repo = trepo.repo
@@ -659,7 +698,7 @@ def test_GitRepo_get_merge_base(src):
 
     # Let's create a detached branch
     branch2 = "_detach_"
-    repo.checkout(branch2, options="--orphan")
+    repo.checkout(branch2, options=["--orphan"])
     # it will have all the files
     # Must not do:  https://github.com/gitpython-developers/GitPython/issues/375
     # repo.git_add('.')
@@ -672,7 +711,7 @@ def test_GitRepo_get_merge_base(src):
     assert(repo.get_merge_base([branch2, branch1]) is None)
 
     # Let's merge them up -- then merge base should match the master
-    repo.merge(branch1)
+    repo.merge(branch1, allow_unrelated=True)
     eq_(repo.get_merge_base(branch1), branch1_hexsha)
 
     # if points to some empty/non-existing branch - should also be None
@@ -719,6 +758,68 @@ def test_get_added_files_commit_msg():
     eq_(f(["f1", "f2"]), 'Added 2 files\n\nFiles:\nf1\nf2')
 
 
+@with_tempfile(mkdir=True)
+@with_tempfile(mkdir=True)
+def test_git_custom_calls(path, path2):
+    # we need a GitRepo instance
+    repo = GitRepo(path, create=True)
+    with open(opj(path, "cc_test.dat"), 'w') as f:
+        f.write("test_git_custom_calls")
+
+    out, err = repo._gitpy_custom_call('add', 'cc_test.dat')
+
+    # actually executed:
+    assert_in("cc_test.dat", repo.get_indexed_files())
+    ok_(repo.dirty)
+
+    # call using cmd_options:
+    out, err = repo._gitpy_custom_call('commit',
+                                       cmd_options={'m': 'added file'})
+    ok_clean_git(path, annex=False)
+    # check output:
+    assert_in("1 file changed", out)
+    assert_in("cc_test.dat", out)
+    eq_('', err)
+
+    # impossible 'add' call should raise ...
+    assert_raises(GitCommandError, repo._gitpy_custom_call,
+                  'add', 'not_existing', expect_fail=False)
+    # .. except we expect it to fail:
+    repo._gitpy_custom_call('add', 'not_existing', expect_fail=True)
+
+    # log outputs:
+    with swallow_logs(new_level=logging.DEBUG) as cm:
+        out, err = repo._gitpy_custom_call('status',
+                                           log_stdout=True,
+                                           log_stderr=True)
+
+        assert_in("On branch master", out)
+        assert_in("nothing to commit", out)
+        eq_("", err)
+        for line in out.splitlines():
+            assert_in("stdout| " + line, cm.out)
+
+    # don't log outputs:
+    with swallow_logs(new_level=logging.DEBUG) as cm:
+        out, err = repo._gitpy_custom_call('status',
+                                           log_stdout=False,
+                                           log_stderr=False)
+
+        assert_in("On branch master", out)
+        assert_in("nothing to commit", out)
+        eq_("", err)
+        eq_("", cm.out)
+
+    # use git_options:
+    # Note: 'path2' doesn't contain a git repository
+    with assert_raises(GitCommandError) as cm:
+        repo._gitpy_custom_call('status', git_options={'C': path2})
+    assert_in("-C %s status" % path2, str(cm.exception))
+    assert_in("fatal: Not a git repository", str(cm.exception))
+
+    # TODO: How to test 'env'?
+
+
 @with_testrepos(flavors=['local'])
 @with_tempfile(mkdir=True)
 def test_get_tracking_branch(o_path, c_path):
@@ -726,7 +827,161 @@ def test_get_tracking_branch(o_path, c_path):
     clone = GitRepo(c_path, o_path)
     eq_(('origin', 'refs/heads/master'), clone.get_tracking_branch())
 
-    clone.checkout('new_branch', '-b')
+    clone.checkout('new_branch', ['-b'])
     eq_((None, None), clone.get_tracking_branch())
 
     eq_(('origin', 'refs/heads/master'), clone.get_tracking_branch('master'))
+
+
+@with_testrepos('submodule_annex', flavors=['clone'])
+def test_submodule_deinit(path):
+
+    top_repo = GitRepo(path, create=False)
+    eq_(['subm 1', 'subm 2'], [s.name for s in top_repo.get_submodules()])
+    top_repo.update_submodule('subm 1', init=True)
+    top_repo.update_submodule('subm 2', init=True)
+    ok_(all([s.module_exists() for s in top_repo.get_submodules()]))
+
+    # modify submodule:
+    with open(opj(top_repo.path, 'subm 1', 'file_ut.dat'), "w") as f:
+        f.write("some content")
+
+    assert_raises(GitCommandError, top_repo.deinit_submodule, 'sub1')
+
+    # using force should work:
+    top_repo.deinit_submodule('subm 1', force=True)
+
+    ok_(not top_repo.repo.submodule('subm 1').module_exists())
+
+
+def test_kwargs_to_options():
+
+    class Some(object):
+
+        @kwargs_to_options(split_single_char_options=True)
+        def f_decorated_split(self, options=None):
+            return options
+
+        @kwargs_to_options(split_single_char_options=False,
+                           target_kw='another')
+        def f_decorated_no_split(self, another=None):
+            return another
+
+    res = Some().f_decorated_split(C="/some/path", m=3, b=True, more_fancy=['one', 'two'])
+    ok_(isinstance(res, list))
+    eq_(res, ['-C', "/some/path", '-b', '-m', '3',
+              '--more-fancy=one', '--more-fancy=two'])
+
+    res = Some().f_decorated_no_split(f='some')
+    eq_(res, ['-fsome'])
+
+
+def test_to_options():
+
+    class Some(object):
+
+        def cmd_func(self, git_options=None, annex_options=None, options=None):
+
+            git_options = git_options[:] if git_options else []
+            annex_options = annex_options[:] if annex_options else []
+            options = options[:] if options else []
+
+            faked_cmd_call = ['git'] + git_options + ['annex'] + \
+                             annex_options + ['my_cmd'] + options
+
+            return faked_cmd_call
+
+    eq_(Some().cmd_func(options=to_options(m="bla", force=True)),
+        ['git', 'annex', 'my_cmd', '--force', '-m', 'bla'])
+
+    eq_(Some().cmd_func(git_options=to_options(C="/some/where"),
+                        annex_options=to_options(JSON=True),
+                        options=to_options(unused=True)),
+        ['git', '-C', '/some/where', 'annex', '--JSON', 'my_cmd', '--unused'])
+
+    eq_(Some().cmd_func(git_options=to_options(C="/some/where", split_single_char_options=False),
+                        annex_options=to_options(JSON=True),
+                        options=to_options(unused=True)),
+        ['git', '-C/some/where', 'annex', '--JSON', 'my_cmd', '--unused'])
+
+
+@with_tempfile
+def test_GitRepo_count_objects(repo_path):
+
+    repo = GitRepo(repo_path, create=True)
+    # test if dictionary returned
+    eq_(isinstance(repo.count_objects, dict), True)
+    # test if dictionary contains keys and values we expect
+    empty_count = {'count': 0, 'garbage': 0,  'in-pack': 0, 'packs': 0, 'prune-packable': 0,
+                   'size': 0, 'size-garbage': 0, 'size-pack': 0}
+    eq_(empty_count, repo.count_objects)
+
+
+@with_tempfile
+def test_get_deleted(path):
+    repo = GitRepo(path, create=True)
+    os.makedirs(opj(path, 'deep'))
+    with open(opj(path, 'test1'), 'w') as f:
+        f.write('some')
+    with open(opj(path, 'deep', 'test2'), 'w') as f:
+        f.write('some more')
+    repo.add('.', commit=True)
+    ok_clean_git(path, annex=False)
+    os.unlink(opj(path, 'test1'))
+    eq_(repo.get_deleted_files(), ['test1'])
+    rmtree(opj(path, 'deep'))
+    eq_(sorted(repo.get_deleted_files()), [opj('deep', 'test2'), 'test1'])
+
+
+@with_tempfile
+def test_optimized_cloning(path):
+    # make test repo with one fiel and one commit
+    originpath = opj(path, 'origin')
+    repo = GitRepo(originpath, create=True)
+    with open(opj(originpath, 'test'), 'w') as f:
+        f.write('some')
+    repo.add('test')
+    repo.commit('init')
+    ok_clean_git(originpath, annex=False)
+    from glob import glob
+
+    def _get_inodes(repo):
+        return dict(
+            [(os.path.join(*o.split(os.sep)[-2:]),
+              os.stat(o).st_ino)
+             for o in glob(os.path.join(repo.repo.git_dir,
+                                        'objects', '*', '*'))])
+
+    origin_inodes = _get_inodes(repo)
+    # now clone it in different ways and see what happens to the object storage
+    from datalad.support.network import get_local_file_url
+    clonepath = opj(path, 'clone')
+    for src in (originpath, get_local_file_url(originpath)):
+        clone = GitRepo(clonepath, url=src, create=True)
+        clone_inodes = _get_inodes(clone)
+        eq_(origin_inodes, clone_inodes, msg='with src={}'.format(src))
+        rmtree(clonepath)
+
+
+@with_tempfile
+@with_tempfile
+def test_GitRepo_gitpy_injection(path, path2):
+
+    gr = GitRepo(path, create=True)
+    gr._GIT_COMMON_OPTIONS.extend(['test-option'])
+
+    with assert_raises(GitCommandError) as cme:
+        gr.repo.git.unknown_git_command()
+    assert_in('test-option', exc_str(cme.exception))
+
+    # once set, these option should be persistent across git calls:
+    with assert_raises(GitCommandError) as cme:
+        gr.repo.git.another_unknown_git_command()
+    assert_in('test-option', exc_str(cme.exception))
+
+    # but other repos should not be affected:
+    gr2 = GitRepo(path2, create=True)
+    with assert_raises(GitCommandError) as cme:
+        gr2.repo.git.unknown_git_command()
+    assert_not_in('test-option', exc_str(cme.exception))
+
