@@ -9,6 +9,7 @@
 
 import logging
 
+from os.path import join as opj
 from collections import OrderedDict
 
 from .utils import eq_, neq_, ok_, nok_, assert_raises
@@ -34,6 +35,8 @@ from ..support.network import is_url
 from ..support.network import is_datalad_compat_ri
 from ..support.network import get_local_file_url
 from ..support.network import is_ssh
+from ..support.network import iso8601_to_epoch
+
 
 def test_same_website():
     ok_(same_website("http://a.b", "http://a.b/2014/01/xxx/"))
@@ -142,11 +145,21 @@ def _check_ri(ri, cls, exact_str=True, localpath=None, **fields):
 
     if localpath:
         eq_(ri_.localpath, localpath)
+        old_localpath = ri_.localpath  # for a test below
     else:
         # if not given -- must be a remote url, should raise exception
         with assert_raises(ValueError):
             ri_.localpath
 
+    # do changes in the path persist?
+    old_str = str(ri_)
+    ri_.path = newpath = opj(ri_.path, 'sub')
+    eq_(ri_.path, newpath)
+    neq_(str(ri_), old_str)
+    if localpath:
+        eq_(ri_.localpath, opj(old_localpath, 'sub'))
+
+
 
 def test_url_base():
     # Basic checks
@@ -276,8 +289,10 @@ def test_url_samples():
         weired_str = 'weired://'
         weired_url = RI(weired_str)
         repr(weired_url)
-        assert_re_in('Parsed version of SSHRI .weired:/. differs from original .weired://.',
-                     cml.out)
+        cml.assert_logged(
+            'Parsed version of SSHRI .weired:/. '
+            'differs from original .weired://.'
+        )
         # but we store original str
         eq_(str(weired_url), weired_str)
         neq_(weired_url.as_str(), weired_str)
@@ -394,9 +409,9 @@ def test_is_ssh():
     ssh_locators = ["ssh://host",
                     "ssh://host/some/where",
                     "user@host:path/sp1",
-                    "user@host:/absloute/path/sp1",
+                    "user@host:/absolute/path/sp1",
                     "host:path/sp1",
-                    "host:/absloute/path/sp1",
+                    "host:/absolute/path/sp1",
                     "user@host"]
     for ri in ssh_locators:
         ok_(is_ssh(ri), "not considered ssh (string): %s" % ri)
@@ -411,3 +426,14 @@ def test_is_ssh():
     for ri in non_ssh_locators:
         ok_(not is_ssh(ri), "considered ssh (string): %s" % ri)
         ok_(not is_ssh(RI(ri)), "considered ssh (RI): %s" % ri)
+
+
+def test_iso8601_to_epoch():
+    epoch = 1467901515
+    eq_(iso8601_to_epoch('2016-07-07T14:25:15+00:00'), epoch)
+    # zone information is actually not used
+    eq_(iso8601_to_epoch('2016-07-07T14:25:15+11:00'), epoch)
+    eq_(iso8601_to_epoch('2016-07-07T14:25:15Z'), epoch)
+    eq_(iso8601_to_epoch('2016-07-07T14:25:15'), epoch)
+
+    eq_(iso8601_to_epoch('2016-07-07T14:25:14'), epoch-1)
\ No newline at end of file
