@@ -12,6 +12,7 @@ import sys
 import os
 import random
 import traceback
+import logging
 
 try:
     # optional direct dependency we might want to kick out
@@ -40,6 +41,7 @@ from .utils import eq_, ok_, assert_false, ok_startswith, nok_startswith, \
     ok_symlink, assert_true, ok_good_symlink, ok_broken_symlink
 
 from .utils import ok_generator
+from .utils import assert_dict_equal
 from .utils import assert_re_in
 from .utils import local_testrepo_flavors
 from .utils import skip_if_no_network
@@ -47,6 +49,7 @@ from .utils import run_under_dir
 from .utils import skip_if
 from .utils import ok_file_has_content
 from .utils import without_http_proxy
+from .utils import with_testsui
 
 #
 # Test with_tempfile, especially nested invocations
@@ -60,7 +63,7 @@ def _with_tempfile_decorated_dummy(path):
 def test_with_tempfile_dir_via_env_variable():
     target = os.path.join(os.path.expanduser("~"), "dataladtesttmpdir")
     assert_false(os.path.exists(target), "directory %s already exists." % target)
-    with patch.dict('os.environ', {'DATALAD_TESTS_TEMPDIR': target}):
+    with patch.dict('os.environ', {'DATALAD_TESTS_TEMP_DIR': target}):
         filename = _with_tempfile_decorated_dummy()
         ok_startswith(filename, target)
 
@@ -100,13 +103,18 @@ def test_nested_with_tempfile_parametrized_surrounded():
 @with_tempfile(content="testtest")
 def test_with_tempfile_content(f):
     ok_file_has_content(f, "testtest")
+    ok_file_has_content(f, "test*", re_=True)
 
 
 def test_with_tempfile_content_raises_on_mkdir():
+
+    @with_tempfile(content="test", mkdir=True)
+    def t():  # pragma: no cover
+        raise AssertionError("must not be run")
+
     with assert_raises(ValueError):
-        @with_tempfile(content="test", mkdir=True)
-        def t():
-            pass
+        # after this commit, it will check when invoking, not when decorating
+        t()
 
 
 def test_with_testrepos():
@@ -144,7 +152,7 @@ def test_with_tempfile_mkdir():
             f.write("TEST LOAD")
 
     check_mkdir()
-    if not os.environ.get('DATALAD_TESTS_KEEPTEMP'):
+    if not os.environ.get('DATALAD_TESTS_TEMP_KEEP'):
         ok_(not os.path.exists(dnames[0]))  # got removed
 
 
@@ -178,7 +186,7 @@ def test_get_most_obscure_supported_name():
 
 def test_keeptemp_via_env_variable():
 
-    if os.environ.get('DATALAD_TESTS_KEEPTEMP'):
+    if os.environ.get('DATALAD_TESTS_TEMP_KEEP'):
         raise SkipTest("We have env variable set to preserve tempfiles")
 
     files = []
@@ -191,7 +199,7 @@ def test_keeptemp_via_env_variable():
     with patch.dict('os.environ', {}):
         check()
 
-    with patch.dict('os.environ', {'DATALAD_TESTS_KEEPTEMP': '1'}):
+    with patch.dict('os.environ', {'DATALAD_TESTS_TEMP_KEEP': '1'}):
         check()
 
     eq_(len(files), 2)
@@ -316,7 +324,7 @@ def test_assert_cwd_unchanged_not_masking_exceptions():
         os.chdir(os.pardir)
         raise ValueError("error exception")
 
-    with swallow_logs() as cml:
+    with swallow_logs(new_level=logging.WARN) as cml:
         with assert_raises(ValueError) as cm:
             do_chdir_value_error()
         # retrospect exception
@@ -336,7 +344,7 @@ def test_assert_cwd_unchanged_not_masking_exceptions():
         os.chdir(os.pardir)
         raise ValueError("error exception")
 
-    with swallow_logs() as cml:
+    with swallow_logs(new_level=logging.WARN) as cml:
         assert_raises(ValueError, do_chdir_value_error)
         eq_(orig_cwd, os.getcwd(),
             "assert_cwd_unchanged didn't return us back to %s" % orig_cwd)
@@ -505,3 +513,44 @@ def test_run_under_dir(d):
     assert_raises(AssertionError, f, 1, 3)
     eq_(getpwd(), orig_pwd)
     eq_(os.getcwd(), orig_cwd)
+
+
+def test_assert_dict_equal():
+    assert_dict_equal({}, {})
+    assert_dict_equal({"a": 3}, {"a": 3})
+    assert_raises(AssertionError, assert_dict_equal, {1: 3}, {1: 4})
+    assert_raises(AssertionError, assert_dict_equal, {1: 3}, {2: 4})
+    assert_raises(AssertionError, assert_dict_equal, {1: 3}, {2: 4, 1: 3})
+    assert_raises(AssertionError, assert_dict_equal, {1: 3}, {2: 4, 1: 'a'})
+    try:
+        import numpy as np
+    except:
+        raise SkipTest("need numpy for this tiny one")
+    # one is scalar another one array
+    assert_raises(AssertionError, assert_dict_equal, {1: 0}, {1: np.arange(1)})
+    assert_raises(AssertionError, assert_dict_equal, {1: 0}, {1: np.arange(3)})
+
+
+def test_testsui():
+    # just one for now to test conflicting arguments
+    with assert_raises(ValueError):
+        @with_testsui(responses='some', interactive=False)
+        def some_func():   # pragma: no cover
+            pass
+
+    from datalad.ui import ui
+
+    @with_testsui(responses=['yes', "maybe so"])
+    def func2(x):
+        assert x == 1
+        eq_(ui.yesno("title"), True)
+        eq_(ui.question("title2"), "maybe so")
+        assert_raises(AssertionError, ui.question, "asking more than we know")
+        return x*2
+    eq_(func2(1), 2)
+
+    @with_testsui(interactive=False)
+    def func3(x):
+        assert_false(ui.is_interactive)
+        return x*3
+    eq_(func3(2), 6)
