@@ -12,11 +12,19 @@
 
 __docformat__ = 'restructuredtext'
 
+from logging import getLogger
+lgr = getLogger('datalad.ui.dialog')
+
+lgr.log(5, "Starting importing ui.dialog")
+
 import os
 import sys
-from six import PY2
+import time
+
 import getpass
-from mock import patch
+
+#!!! OPT adds >100ms to import time!!!
+# from mock import patch
 from collections import deque
 from copy import copy
 
@@ -32,18 +40,18 @@ from .base import InteractiveUI
 # - docker has multiple simultaneous progressbars.  Apparently "navigation"
 #   is obtained with escape characters in the terminal.
 #   see docker/pkg/jsonmessage/jsonmessage.go or following snippet
-"""
-from time import sleep
-import sys
-
-out = sys.stderr
-for i in range(10):
-  diff = 2
-  if i:
-      out.write("%c[%dA" % (27, diff))
-  out.write("%d\n%d\n" % (i, i ** 2))
-  sleep(0.5)
-"""
+#
+#from time import sleep
+#import sys
+#
+#out = sys.stderr
+#for i in range(10):
+#  diff = 2
+#  if i:
+#      out.write("%c[%dA" % (27, diff))
+#  out.write("%d\n%d\n" % (i, i ** 2))
+#  sleep(0.5)
+#
 # They also use JSON representation for the message which might provide a nice abstraction
 # Other useful codes
 #         // <ESC>[2K = erase entire current line
@@ -53,7 +61,6 @@ for i in range(10):
 # reference for ESC codes: http://ascii-table.com/ansi-escape-sequences.php
 
 
-
 @auto_repr
 class ConsoleLog(object):
 
@@ -92,34 +99,55 @@ class ConsoleLog(object):
             pbar = progressbars[backend]
         return pbar(*args, out=self.out, **kwargs)
 
+    @property
+    def is_interactive(self):
+        return isinstance(self, InteractiveUI)
 
-def getpass_echo(*args, **kwargs):
+
+def getpass_echo(prompt='Password: ', stream=None):
     """Q&D workaround until we have proper 'centralized' UI -- just use getpass BUT enable echo
     """
     if on_windows:
         # Can't do anything fancy yet, so just ask the one without echo
-        return getpass.getpass(*args, **kwargs)
+        return getpass.getpass(prompt=prompt, stream=stream)
     else:
         # We can mock patch termios so that ECHO is not turned OFF.
         # Side-effect -- additional empty line is printed
-        with patch('termios.ECHO', 255**2):
-            return getpass.getpass(*args, **kwargs)
+
+        # def _no_emptyline_write(out):
+        #     # Additional mock to prevent not needed empty line print since we do have echo
+        #     # doesn't work since we don't know the stream here really
+        #     if out == '\n':
+        #         return
+        #     stream.write(out)
+        from mock import patch
+        with patch('termios.ECHO', 255 ** 2):
+            #patch.object(stream, 'write', _no_emptyline_write(stream)):
+            return getpass.getpass(prompt=prompt, stream=stream)
 
 
 @auto_repr
 class DialogUI(ConsoleLog, InteractiveUI):
 
+    def __init__(self, *args, **kwargs):
+        super(DialogUI, self).__init__(*args, **kwargs)
+        # ATM doesn't make sense to print the same title for subsequent questions
+        # so we will store previous one and not show it if was the previous one shown
+        # within 5 seconds from prev question
+        self._prev_title = None
+        self._prev_title_time = 0
+
     def question(self, text,
                  title=None, choices=None,
                  default=None,
                  hidden=False):
         # Do initial checks first
-        if default and default not in choices:
+        if default and choices and default not in choices:
             raise ValueError("default value %r is not among choices: %s"
                              % (default, choices))
 
         msg = ''
-        if title:
+        if title and not (title == self._prev_title and time.time() - self._prev_title_time < 5):
             # might not actually get displayed if all in/out redirected
             # self.out.write(title + "\n")
             # so merge into msg for getpass
@@ -132,14 +160,15 @@ class DialogUI(ConsoleLog, InteractiveUI):
 
         if choices is not None:
             msg += "%s (choices: %s)" % (text, ', '.join(map(mark_default, choices)))
+        elif default is not None:
+            msg += '{} [{}]'.format(text, default)
         else:
             msg += text
-        """
-        Anaconda format:
-
-Question? [choice1|choice2]
-[default] >>> yes
-        """
+        # Like this:
+        #Anaconda format:
+        #
+        #Question? [choice1|choice2]
+        #[default] >>> yes
         attempt = 0
         while True:
             attempt += 1
@@ -166,13 +195,17 @@ Question? [choice1|choice2]
                            % (response, choices))
                 continue
             break
+
+        self._prev_title = title
+        self._prev_title_time = time.time()
+
         return response
 
 
 # poor man thingie for now
 @auto_repr
 class UnderAnnexUI(DialogUI):
-    def __init__(self, **kwargs):
+    def __init__(self, specialremote=None, **kwargs):
         if 'out' not in kwargs:
             # to avoid buffering
             # http://stackoverflow.com/a/181654/1265472
@@ -180,6 +213,18 @@ class UnderAnnexUI(DialogUI):
             # but wasn't effective! sp kist straogjt for now
             kwargs['out'] = sys.stderr
         super(UnderAnnexUI, self).__init__(**kwargs)
+        self.specialremote = specialremote
+
+    def set_specialremote(self, specialremote):
+        self.specialremote = specialremote
+
+    def get_progressbar(self, *args, **kwargs):
+        if self.specialremote:
+            kwargs = kwargs.copy()
+            kwargs['backend'] = 'annex-remote'
+            kwargs['remote'] = self.specialremote
+        return super(UnderAnnexUI, self).get_progressbar(
+                *args, **kwargs)
 
 
 @auto_repr
@@ -226,3 +271,5 @@ class UnderTestsUI(DialogUI):
         self.clear_responses()
         assert not len(responses), \
             "Still have some responses left: %s" % repr(self._responses)
+
+lgr.log(5, "Done importing ui.dialog")
