@@ -8,14 +8,11 @@
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 
 import collections
+import hashlib
 import re
 import six.moves.builtins as __builtin__
 import time
 
-from os.path import curdir, basename, exists, realpath, islink, join as opj, isabs, normpath, expandvars, expanduser, abspath
-from six.moves.urllib.parse import quote as urlquote, unquote as urlunquote, urlsplit
-from six import text_type
-
 import logging
 import shutil
 import stat
@@ -24,9 +21,25 @@ import sys
 import tempfile
 import platform
 import gc
+import glob
 
+from contextlib import contextmanager
 from functools import wraps
 from time import sleep
+from inspect import getargspec
+
+from os.path import sep as dirsep
+from os.path import commonprefix
+from os.path import curdir, basename, exists, realpath, islink, join as opj
+from os.path import isabs, normpath, expandvars, expanduser, abspath, sep
+from os.path import isdir
+from os.path import relpath
+
+
+from six import text_type, binary_type
+
+# from datalad.dochelpers import get_docstring_split
+from datalad.consts import TIMESTAMP_FMT
 
 lgr = logging.getLogger("datalad.utils")
 
@@ -34,29 +47,43 @@ lgr.log(5, "Importing datalad.utils")
 #
 # Some useful variables
 #
-_platform_system = platform.system().lower()
-on_windows = _platform_system == 'windows'
-on_osx = _platform_system == 'darwin'
-on_linux = _platform_system == 'linux'
+platform_system = platform.system().lower()
+on_windows = platform_system == 'windows'
+on_osx = platform_system == 'darwin'
+on_linux = platform_system == 'linux'
 try:
-    linux_distribution = platform.linux_distribution()
+    linux_distribution_name, linux_distribution_release \
+        = platform.linux_distribution()[:2]
     on_debian_wheezy = on_linux \
-                       and linux_distribution[0] == 'debian' \
-                       and linux_distribution[1].startswith('7.')
+                       and linux_distribution_name == 'debian' \
+                       and linux_distribution_release.startswith('7.')
 except:  # pragma: no cover
+    # MIH: IndexError?
     on_debian_wheezy = False
+    linux_distribution_name = linux_distribution_release = None
 
 #
 # Little helpers
 #
 
 
-def assure_tuple_or_list(obj):
-    """Given an object, wrap into a tuple if not list or tuple
+def get_func_kwargs_doc(func):
+    """ Provides args for a function
+
+    Parameters
+    ----------
+    func: str
+      name of the function from which args are being requested
+
+    Returns
+    -------
+    list
+      of the args that a function takes in
     """
-    if isinstance(obj, list) or isinstance(obj, tuple):
-        return obj
-    return (obj,)
+    return getargspec(func)[0]
+
+    # TODO: format error message with descriptions of args
+    # return [repr(dict(get_docstring_split(func)[1]).get(x)) for x in getargspec(func)[0]]
 
 
 def any_re_search(regexes, value):
@@ -81,7 +108,7 @@ def shortened_repr(value, l=30):
         if hasattr(value, '__repr__') and (value.__repr__ is not object.__repr__):
             value_repr = repr(value)
             if not value_repr.startswith('<') and len(value_repr) > l:
-                value_repr = "<<%s...>>" % (value_repr[:l-8])
+                value_repr = "<<%s...>>" % (value_repr[:l - 8])
             elif value_repr.startswith('<') and value_repr.endswith('>') and ' object at 0x':
                 raise ValueError("I hate those useless long reprs")
         else:
@@ -111,6 +138,7 @@ def __auto_repr__(obj):
 
     return "%s(%s)" % (obj.__class__.__name__, ', '.join(items))
 
+
 def auto_repr(cls):
     """Decorator for a class to assign it an automagic quick and dirty __repr__
 
@@ -122,28 +150,31 @@ def auto_repr(cls):
     cls.__repr__ = __auto_repr__
     return cls
 
+
 def is_interactive():
     """Return True if all in/outs are tty"""
     # TODO: check on windows if hasattr check would work correctly and add value:
     #
     return sys.stdin.isatty() and sys.stdout.isatty() and sys.stderr.isatty()
 
-import hashlib
+
 def md5sum(filename):
     with open(filename, 'rb') as f:
         return hashlib.md5(f.read()).hexdigest()
 
+
 def sorted_files(dout):
     """Return a (sorted) list of files under dout
     """
-    return sorted(sum([[opj(r, f)[len(dout)+1:] for f in files]
-                       for r,d,files in os.walk(dout)
+    return sorted(sum([[opj(r, f)[len(dout) + 1:] for f in files]
+                       for r, d, files in os.walk(dout)
                        if not '.git' in r], []))
 
-from os.path import sep as dirsep
-_VCS_REGEX = '%s\.(git|gitattributes|svn|bzr|hg)(?:%s|$)' % (dirsep, dirsep)
+_VCS_REGEX = '%s\.(?:git|gitattributes|svn|bzr|hg)(?:%s|$)' % (dirsep, dirsep)
+_DATALAD_REGEX = '%s\.(?:datalad)(?:%s|$)' % (dirsep, dirsep)
 
-def find_files(regex, topdir=curdir, exclude=None, exclude_vcs=True, dirs=False):
+
+def find_files(regex, topdir=curdir, exclude=None, exclude_vcs=True, exclude_datalad=False, dirs=False):
     """Generator to find files matching regex
 
     Parameters
@@ -153,12 +184,15 @@ def find_files(regex, topdir=curdir, exclude=None, exclude_vcs=True, dirs=False)
       Matches to exclude
     exclude_vcs:
       If True, excludes commonly known VCS subdirectories.  If string, used
-      as regex to exclude those files (regex: %r)
+      as regex to exclude those files (regex: `%r`)
+    exclude_datalad:
+      If True, excludes files known to be datalad meta-data files (e.g. under
+      .datalad/ subdirectory) (regex: `%r`)
     topdir: basestring, optional
       Directory where to search
     dirs: bool, optional
       Either to match directories as well as files
-    """ % _VCS_REGEX
+    """
 
     for dirpath, dirnames, filenames in os.walk(topdir):
         names = (dirnames + filenames) if dirs else filenames
@@ -170,7 +204,10 @@ def find_files(regex, topdir=curdir, exclude=None, exclude_vcs=True, dirs=False)
                 continue
             if exclude_vcs and re.search(_VCS_REGEX, path):
                 continue
+            if exclude_datalad and re.search(_DATALAD_REGEX, path):
+                continue
             yield path
+find_files.__doc__ %= (_VCS_REGEX, _DATALAD_REGEX)
 
 
 def expandpath(path, force_absolute=True):
@@ -195,6 +232,7 @@ def is_explicit_path(path):
         or path.startswith(os.curdir + os.sep) \
         or path.startswith(os.pardir + os.sep)
 
+
 def rotree(path, ro=True, chmod_files=True):
     """To make tree read-only or writable
 
@@ -250,10 +288,10 @@ def rmtree(path, chmod_files='auto', *args, **kwargs):
 def rmtemp(f, *args, **kwargs):
     """Wrapper to centralize removing of temp files so we could keep them around
 
-    It will not remove the temporary file/directory if DATALAD_TESTS_KEEPTEMP
+    It will not remove the temporary file/directory if DATALAD_TESTS_TEMP_KEEP
     environment variable is defined
     """
-    if not os.environ.get('DATALAD_TESTS_KEEPTEMP'):
+    if not os.environ.get('DATALAD_TESTS_TEMP_KEEP'):
         if not os.path.lexists(f):
             lgr.debug("Path %s does not exist, so can't be removed" % f)
             return
@@ -284,10 +322,11 @@ def file_basename(name, return_ext=False):
     bname = basename(name)
     fbname = re.sub('(\.[a-zA-Z_]\S{1,4}){0,2}$', '', bname)
     if return_ext:
-        return fbname, bname[len(fbname)+1:]
+        return fbname, bname[len(fbname) + 1:]
     else:
         return fbname
 
+
 def escape_filename(filename):
     """Surround filename in "" and escape " in the filename
     """
@@ -295,6 +334,7 @@ def escape_filename(filename):
     filename = '"%s"' % filename
     return filename
 
+
 def encode_filename(filename):
     """Encode unicode filename
     """
@@ -332,6 +372,33 @@ else:
         # doesn't work on OSX
         # Runner().run(['touch', '-h', '-d', '@%s' % mtime, filepath])
 
+
+def assure_tuple_or_list(obj):
+    """Given an object, wrap into a tuple if not list or tuple
+    """
+    if isinstance(obj, list) or isinstance(obj, tuple):
+        return obj
+    return (obj,)
+
+
+def assure_list(s):
+    """Given not a list, would place it into a list. If None - empty list is returned
+
+    Parameters
+    ----------
+    s: list or anything
+    """
+
+    if isinstance(s, list):
+        return s
+    elif isinstance(s, tuple):
+        return list(s)
+    elif s is None:
+        return []
+    else:
+        return [s]
+
+
 def assure_list_from_str(s, sep='\n'):
     """Given a multiline string convert it to a list of return None if empty
 
@@ -370,12 +437,69 @@ def assure_dict_from_str(s, **kwargs):
             raise ValueError("{} is not in key=value format".format(repr(value_str)))
         k, v = value_str.split('=', 1)
         if k in out:
-            err  = "key {} was already defined in {}, but new value {} was provided".format(k, out, v)
+            err = "key {} was already defined in {}, but new value {} was provided".format(k, out, v)
             raise ValueError(err)
         out[k] = v
     return out
 
 
+def assure_unicode(s, encoding='utf-8'):
+    """Convert/decode to unicode (PY2) or str (PY3) if of 'binary_type'"""
+    return s.decode(encoding) if isinstance(s, binary_type) else s
+
+
+def unique(seq, key=None):
+    """Given a sequence return a list only with unique elements while maintaining order
+
+    This is the fastest solution.  See
+    https://www.peterbe.com/plog/uniqifiers-benchmark
+    and
+    http://stackoverflow.com/a/480227/1265472
+    for more information.
+    Enhancement -- added ability to compare for uniqueness using a key function
+
+    Parameters
+    ----------
+    seq:
+      Sequence to analyze
+    key: callable, optional
+      Function to call on each element so we could decide not on a full
+      element, but on its member etc
+    """
+    seen = set()
+    seen_add = seen.add
+    if not key:
+        return [x for x in seq if not (x in seen or seen_add(x))]
+    else:
+        # OPT: could be optimized, since key is called twice, but for our cases
+        # should be just as fine
+        return [x for x in seq if not (key(x) in seen or seen_add(key(x)))]
+
+
+#
+# Generators helpers
+#
+
+def saved_generator(gen):
+    """Given a generator returns two generators, where 2nd one just replays
+
+    So the first one would be going through the generated items and 2nd one
+    would be yielding saved items
+    """
+    saved = []
+
+    def gen1():
+        for x in gen:  # iterating over original generator
+            saved.append(x)
+            yield x
+
+    def gen2():
+        for x in saved:  # yielding saved entries
+            yield x
+
+    return gen1(), gen2()
+
+
 #
 # Decorators
 #
@@ -410,27 +534,30 @@ def optional_args(decorator):
 
 
 # TODO: just provide decorators for tempfile.mk* functions. This is ugly!
-def get_tempfile_kwargs(tkwargs={}, prefix="", wrapped=None):
+def get_tempfile_kwargs(tkwargs=None, prefix="", wrapped=None):
     """Updates kwargs to be passed to tempfile. calls depending on env vars
     """
-    # operate on a copy of tkwargs to avoid any side-effects
-    tkwargs_ = tkwargs.copy()
+    if tkwargs is None:
+        tkwargs_ = {}
+    else:
+        # operate on a copy of tkwargs to avoid any side-effects
+        tkwargs_ = tkwargs.copy()
 
     # TODO: don't remember why I had this one originally
     # if len(targs)<2 and \
-    if not 'prefix' in tkwargs_:
+    if 'prefix' not in tkwargs_:
         tkwargs_['prefix'] = '_'.join(
             ['datalad_temp'] +
             ([prefix] if prefix else []) +
-            ([''] if (on_windows or not wrapped)
-                  else [wrapped.__name__]))
+            ([''] if (on_windows or not wrapped) else [wrapped.__name__]))
 
-    directory = os.environ.get('DATALAD_TESTS_TEMPDIR')
+    directory = os.environ.get('DATALAD_TESTS_TEMP_DIR')
     if directory and 'dir' not in tkwargs_:
         tkwargs_['dir'] = directory
 
     return tkwargs_
 
+
 @optional_args
 def line_profile(func):
     """Q&D helper to line profile the function and spit out stats
@@ -451,7 +578,12 @@ def line_profile(func):
 # Context Managers
 #
 
-from contextlib import contextmanager
+
+@contextmanager
+def nothing_cm():
+    """Just a dummy cm to programmically switch context managers"""
+    yield
+
 
 @contextmanager
 def swallow_outputs():
@@ -467,7 +599,6 @@ def swallow_outputs():
     print function had desired effect
     """
 
-    debugout = sys.stdout
     class StringIOAdapter(object):
         """Little adapter to help getting out/err values
         """
@@ -506,8 +637,6 @@ def swallow_outputs():
             rmtemp(out_name)
             rmtemp(err_name)
 
-
-
     def fake_print(*args, **kwargs):
         sep = kwargs.pop('sep', ' ')
         end = kwargs.pop('end', '\n')
@@ -540,7 +669,7 @@ def swallow_outputs():
 
 
 @contextmanager
-def swallow_logs(new_level=None):
+def swallow_logs(new_level=None, file_=None):
     """Context manager to consume all logs.
 
     """
@@ -558,10 +687,14 @@ def swallow_logs(new_level=None):
         And to stay consistent with how swallow_outputs behaves
         """
         def __init__(self):
-            kw = dict()
-            get_tempfile_kwargs(kw, prefix="logs")
-
-            self._out = open(tempfile.mktemp(**kw), 'w')
+            if file_ is None:
+                kw = get_tempfile_kwargs({}, prefix="logs")
+                out_file = tempfile.mktemp(**kw)
+            else:
+                out_file = file_
+            # PY3 requires clearly one or another.  race condition possible
+            self._out = open(out_file, 'a')
+            self._final_out = None
 
         def _read(self, h):
             with open(h.name) as f:
@@ -569,8 +702,12 @@ def swallow_logs(new_level=None):
 
         @property
         def out(self):
-            self._out.flush()
-            return self._read(self._out)
+            if self._final_out is not None:
+                # we closed and cleaned up already
+                return self._final_out
+            else:
+                self._out.flush()
+                return self._read(self._out)
 
         @property
         def lines(self):
@@ -581,16 +718,62 @@ def swallow_logs(new_level=None):
             return self._out
 
         def cleanup(self):
+            # store for access while object exists
+            self._final_out = self.out
             self._out.close()
             out_name = self._out.name
             del self._out
             gc.collect()
-            rmtemp(out_name)
+            if not file_:
+                rmtemp(out_name)
+
+        def assert_logged(self, msg=None, level=None, regex=True, **kwargs):
+            """Provide assertion on either a msg was logged at a given level
+
+            If neither `msg` nor `level` provided, checks if anything was logged
+            at all.
+
+            Parameters
+            ----------
+            msg: str, optional
+              Message (as a regular expression, if `regex`) to be searched.
+              If no msg provided, checks if anything was logged at a given level.
+            level: str, optional
+              String representing the level to be logged
+            regex: bool, optional
+              If False, regular `assert_in` is used
+            **kwargs: str, optional
+              Passed to `assert_re_in` or `assert_in`
+            """
+            from datalad.tests.utils import assert_re_in
+            from datalad.tests.utils import assert_in
+
+            if regex:
+                match = '\[%s\] ' % level if level else "\[\S+\] "
+            else:
+                match = '[%s] ' % level if level else ''
+
+            if msg:
+                match += msg
+
+            if match:
+                (assert_re_in if regex else assert_in)(match, self.out, **kwargs)
+            else:
+                assert not kwargs, "no kwargs to be passed anywhere"
+                assert self.out, "Nothing was logged!?"
 
     adapter = StringIOAdapter()
-    lgr.handlers = [logging.StreamHandler(adapter.handle)]
+    # TODO: it does store messages but without any formatting, i.e. even without
+    # date/time prefix etc.  IMHO it should preserve formatting in case if file_ is
+    # set
+    swallow_handler = logging.StreamHandler(adapter.handle)
+    # we want to log levelname so we could test against it
+    swallow_handler.setFormatter(
+        logging.Formatter('[%(levelname)s] %(message)s'))
+    lgr.handlers = [swallow_handler]
     if old_level < logging.DEBUG:  # so if HEAVYDEBUG etc -- show them!
         lgr.handlers += old_handlers
+
     if isinstance(new_level, str):
         new_level = getattr(logging, new_level)
 
@@ -599,6 +782,8 @@ def swallow_logs(new_level=None):
 
     try:
         yield adapter
+        # TODO: if file_ and there was an exception -- most probably worth logging it?
+        # although ideally it should be the next log outside added to that file_ ... oh well
     finally:
         lgr.handlers, lgr.level = old_handlers, old_level
         adapter.cleanup()
@@ -607,8 +792,10 @@ def swallow_logs(new_level=None):
 #
 # Additional handlers
 #
-_sys_excepthook = sys.excepthook # Just in case we ever need original one
-def setup_exceptionhook():
+_sys_excepthook = sys.excepthook  # Just in case we ever need original one
+
+
+def setup_exceptionhook(ipython=False):
     """Overloads default sys.excepthook with our exceptionhook handler.
 
        If interactive, our exceptionhook handler will invoke
@@ -616,19 +803,20 @@ def setup_exceptionhook():
     """
 
     def _datalad_pdb_excepthook(type, value, tb):
+        import traceback
+        traceback.print_exception(type, value, tb)
+        print()
         if is_interactive():
-            import traceback, pdb
-            traceback.print_exception(type, value, tb)
-            print()
+            import pdb
             pdb.post_mortem(tb)
-        else:
-            lgr.warn("We cannot setup exception hook since not in interactive mode")
-            # we are in interactive mode or we don't have a tty-like
-            # device, so we call the default hook
-            #sys.__excepthook__(type, value, tb)
-            _sys_excepthook(type, value, tb)
 
-    sys.excepthook = _datalad_pdb_excepthook
+    if ipython:
+        from IPython.core import ultratb
+        sys.excepthook = ultratb.FormattedTB(mode='Verbose',
+                                             # color_scheme='Linux',
+                                             call_pdb=is_interactive())
+    else:
+        sys.excepthook = _datalad_pdb_excepthook
 
 
 def assure_dir(*args):
@@ -642,6 +830,7 @@ def assure_dir(*args):
         os.makedirs(dirname)
     return dirname
 
+
 def updated(d, update):
     """Return a copy of the input with the 'update'
 
@@ -651,6 +840,7 @@ def updated(d, update):
     d.update(update)
     return d
 
+
 def getpwd():
     """Try to return a CWD without dereferencing possible symlinks
 
@@ -661,6 +851,7 @@ def getpwd():
     except KeyError:
         return os.getcwd()
 
+
 class chpwd(object):
     """Wrapper around os.chdir which also adjusts environ['PWD']
 
@@ -697,12 +888,48 @@ class chpwd(object):
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         if self._prev_pwd:
-            chpwd(self._prev_pwd, logsuffix="(coming back)")
+            # Need to use self.__class__ so this instance, if the entire
+            # thing mocked during the test, still would use correct chpwd
+            self.__class__(self._prev_pwd, logsuffix="(coming back)")
+
+
+def with_pathsep(path):
+    """Little helper to guarantee that path ends with /"""
+    return path + sep if not path.endswith(sep) else path
+
+
+def get_path_prefix(path, pwd=None):
+    """Get path prefix (for current directory)
+
+    Returns relative path to the topdir, if we are under topdir, and if not
+    absolute path to topdir.  If `pwd` is not specified - current directory
+    assumed
+    """
+    pwd = pwd or getpwd()
+    if not isabs(path):
+        # if not absolute -- relative to pwd
+        path = opj(getpwd(), path)
+    path_ = with_pathsep(path)
+    pwd_ = with_pathsep(pwd)
+    common = commonprefix((path_, pwd_))
+    if common.endswith(sep) and common in {path_, pwd_}:
+        # we are in subdir or above the path = use relative path
+        location_prefix = relpath(path, pwd)
+        # if benign "here" - cut off
+        if location_prefix in (curdir, curdir + sep):
+            location_prefix = ''
+        return location_prefix
+    else:
+        # just return absolute path
+        return path
 
 
 def knows_annex(path):
     """Returns whether at a given path there is information about an annex
 
+    It is just a thin wrapper around GitRepo.is_with_annex() classmethod
+    which also checks for `path` to exist first.
+
     This includes actually present annexes, but also uninitialized ones, or
     even the presence of a remote annex branch.
     """
@@ -711,9 +938,118 @@ def knows_annex(path):
         lgr.debug("No annex: test path {0} doesn't exist".format(path))
         return False
     from datalad.support.gitrepo import GitRepo
-    repo = GitRepo(path, create=False)
-    return "origin/git-annex" in repo.get_remote_branches() \
-           or "git-annex" in repo.get_branches()
+    return GitRepo(path, init=False, create=False).is_with_annex()
+
+
+@contextmanager
+def make_tempfile(content=None, wrapped=None, **tkwargs):
+    """Helper class to provide a temporary file name and remove it at the end (context manager)
+
+    Parameters
+    ----------
+    mkdir : bool, optional (default: False)
+        If True, temporary directory created using tempfile.mkdtemp()
+    content : str or bytes, optional
+        Content to be stored in the file created
+    wrapped : function, optional
+        If set, function name used to prefix temporary file name
+    `**tkwargs`:
+        All other arguments are passed into the call to tempfile.mk{,d}temp(),
+        and resultant temporary filename is passed as the first argument into
+        the function t.  If no 'prefix' argument is provided, it will be
+        constructed using module and function names ('.' replaced with
+        '_').
+
+    To change the used directory without providing keyword argument 'dir' set
+    DATALAD_TESTS_TEMP_DIR.
+
+    Examples
+    --------
+        >>> from os.path import exists
+        >>> from datalad.utils import make_tempfile
+        >>> with make_tempfile() as fname:
+        ...    k = open(fname, 'w').write('silly test')
+        >>> assert not exists(fname)  # was removed
+
+        >>> with make_tempfile(content="blah") as fname:
+        ...    assert open(fname).read() == "blah"
+    """
+
+    if tkwargs.get('mkdir', None) and content is not None:
+        raise ValueError("mkdir=True while providing content makes no sense")
+
+    tkwargs_ = get_tempfile_kwargs(tkwargs, wrapped=wrapped)
+
+    # if DATALAD_TESTS_TEMP_DIR is set, use that as directory,
+    # let mktemp handle it otherwise. However, an explicitly provided
+    # dir=... will override this.
+    mkdir = tkwargs_.pop('mkdir', False)
+
+    filename = {False: tempfile.mktemp,
+                True: tempfile.mkdtemp}[mkdir](**tkwargs_)
+    filename = realpath(filename)
+
+    if content:
+        with open(filename, 'w' + ('b' if isinstance(content, binary_type) else '')) as f:
+            f.write(content)
+
+    if __debug__:
+        # TODO mkdir
+        lgr.debug('Created temporary thing named %s"' % filename)
+    try:
+        yield filename
+    finally:
+        # glob here for all files with the same name (-suffix)
+        # would be useful whenever we requested .img filename,
+        # and function creates .hdr as well
+        lsuffix = len(tkwargs_.get('suffix', ''))
+        filename_ = lsuffix and filename[:-lsuffix] or filename
+        filenames = glob.glob(filename_ + '*')
+        if len(filename_) < 3 or len(filenames) > 5:
+            # For paranoid yoh who stepped into this already ones ;-)
+            lgr.warning("It is unlikely that it was intended to remove all"
+                        " files matching %r. Skipping" % filename_)
+            return
+        for f in filenames:
+            try:
+                rmtemp(f)
+            except OSError:
+                pass
+
+
+def _path_(*p):
+    """Given a path in POSIX" notation, regenerate one in native to the env one"""
+    if on_windows:
+        return opj(*map(lambda x: x.split('/'), p))
+    else:
+        # Assume that all others as POSIX compliant so nothing to be done
+        return opj(*p)
+
+
+def get_timestamp_suffix(time_=None, prefix='-'):
+    """Return a time stamp (full date and time up to second)
+
+    primarily to be used for generation of log files names
+    """
+    args = []
+    if time_ is not None:
+        if isinstance(time_, int):
+            time_ = time.gmtime(time_)
+        args.append(time_)
+    return time.strftime(prefix + TIMESTAMP_FMT, *args)
+
+
+def get_logfilename(dspath, cmd='datalad'):
+    """Return a filename to use for logging under a dataset/repository
+
+    directory would be created if doesn't exist, but dspath must exist
+    and be a directory
+    """
+    assert(exists(dspath))
+    assert(isdir(dspath))
+    ds_logdir = assure_dir(dspath, '.git', 'datalad', 'logs')  # TODO: use WEB_META_LOG whenever #789 merged
+    return opj(ds_logdir, 'crawl-%s.log' % get_timestamp_suffix())
+
 
 lgr.log(5, "Done importing datalad.utils")
 
